=== modified file 'NEWS'
--- NEWS	2007-02-08 15:00:47 +0000
+++ NEWS	2007-02-14 09:51:20 +0000
@@ -22,6 +22,9 @@
     * ``bzr help global-options`` describes the global options.
       (Aaron Bentley)
 
+    * ``bzr pull --overwrite`` will now correctly overwrite checkouts.
+      (Robert Collins)
+
     * Files are now allowed to change kind (e.g. from file to symlink).
       Supported by ``commit``, ``revert`` and ``status``
       (Aaron Bentley)
@@ -29,16 +32,27 @@
     * ``inventory`` and ``unknowns`` hidden in favour of ``ls``
       (Aaron Bentley)
 
+    * ``bzr help checkouts`` descibes what checkouts are and some possible
+      uses of them. (James Westby, Aaron Bentley)
+
+    * A new ``-d`` option to push, pull and merge overrides the default 
+      directory.  (Martin Pool)
+
   INTERNALS:
 
     * Reserved ids (any revision-id ending in a colon) are rejected by
       versionedfiles, repositories, branches, and working trees
       (Aaron Bentley)
 
-    * New Branch hooks facility, with one initial hook 'set_rh' which triggers
-      whenever the revision history is set. This allows triggering on e.g.
-      push, pull, commit, and so on. Developed for use with the branchrss
-      plugin. See bzrlib.branch.BranchHooks for more details. (Robert Collins)
+    * Minor performance improvement by not creating a ProgressBar for
+      every KnitIndex we create. (about 90ms for a bzr.dev tree)
+      (John Arbash Meinel)
+
+    * New easier to use Branch hooks facility. There are five initial hooks,
+      all documented in bzrlib.branch.BranchHooks.__init__ - 'set_rh',
+      'post_push', 'post_pull', 'post_commit', 'post_uncommit'. These hooks
+      fire after the matching operation on a branch has taken place, and were
+      originally added for the branchrss plugin. (Robert Collins)
 
     * New method ``Branch.push()`` which should be used when pushing from a
       branch as it makes performance and policy decisions to match the UI
@@ -62,6 +76,10 @@
       continues to work, but callers accessed the FORMATTERS dictionary
       directly will not.
 
+    * ``GPGStrategy.sign()`` will now raise ``BzrBadParameterUnicode`` if
+      you pass a Unicode string rather than an 8-bit string. Callers need
+      to be updated to encode first. (John Arbash Meinel)
+
   BUGFIXES:
 
     * ``bzr annotate`` now uses dotted revnos from the viewpoint of the
@@ -71,6 +89,18 @@
     * Lock operations no longer hang if they encounter a permission problem.
       (Aaron Bentley)
 
+    * ``bzr push`` can resume a push that was canceled before it finished.
+      Also, it can push even if the target directory exists if you supply
+      the ``--use-existing-dir`` flag.
+      (John Arbash Meinel, #30576, #45504)
+
+    * Fix http proxy authentication when user and an optional
+      password appears in the *_proxy vars. (Vincent Ladeuil,
+      #83954).
+
+    * ``bzr log branch/file`` works for local treeless branches
+      (Aaron Bentley, #84247)
+
   TESTING:
 
     * New ``--first`` option to ``bzr selftest`` to run specified tests

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2007-02-03 10:02:11 +0000
+++ bzrlib/branch.py	2007-02-13 11:48:14 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -259,7 +259,7 @@
         if config is None:
             config = self.get_config()
         
-        return self.repository.get_commit_builder(self, parents, config, 
+        return self.repository.get_commit_builder(self, parents, config,
             timestamp, timezone, committer, revprops, revision_id)
 
     def get_master_branch(self):
@@ -612,9 +612,9 @@
         Weaves are used if this branch's repostory uses weaves.
         """
         if isinstance(self.bzrdir, bzrdir.BzrDirPreSplitOut):
-            from bzrlib import repository
+            from bzrlib.repofmt import weaverepo
             format = bzrdir.BzrDirMetaFormat1()
-            format.repository_format = repository.RepositoryFormat7()
+            format.repository_format = weaverepo.RepositoryFormat7()
         else:
             format = self.repository.bzrdir.cloning_metadir()
         return format
@@ -750,11 +750,39 @@
         notified.
         """
         dict.__init__(self)
+        # Introduced in 0.15:
         # invoked whenever the revision history has been set
         # with set_revision_history. The api signature is
         # (branch, revision_history), and the branch will
-        # be write-locked. Introduced in 0.15.
+        # be write-locked.
         self['set_rh'] = []
+        # invoked after a push operation completes.
+        # the api signature is
+        # (source, local, master, old_revno, old_revid, new_revno, new_revid)
+        # where local is the local branch or None, master is the target 
+        # master branch, and the rest should be self explanatory. The source
+        # is read locked and the target branches write locked. Source will
+        # be the local low-latency branch.
+        self['post_push'] = []
+        # invoked after a pull operation completes.
+        # the api signature is
+        # (source, local, master, old_revno, old_revid, new_revno, new_revid)
+        # where local is the local branch or None, master is the target 
+        # master branch, and the rest should be self explanatory. The source
+        # is read locked and the target branches write locked. The local
+        # branch is the low-latency branch.
+        self['post_pull'] = []
+        # invoked after a commit operation completes.
+        # the api signature is 
+        # (local, master, old_revno, old_revid, new_revno, new_revid)
+        # old_revid is NULL_REVISION for the first commit to a branch.
+        self['post_commit'] = []
+        # invoked after a uncommit operation completes.
+        # the api signature is
+        # (local, master, old_revno, old_revid, new_revno, new_revid) where
+        # local is the local branch or None, master is the target branch,
+        # and an empty branch recieves new_revno of 0, new_revid of None.
+        self['post_uncommit'] = []
 
     def install_hook(self, hook_name, a_callable):
         """Install a_callable in to the hook hook_name.
@@ -1249,11 +1277,18 @@
         return self.bzrdir.open_workingtree()
 
     @needs_write_lock
-    def pull(self, source, overwrite=False, stop_revision=None):
-        """See Branch.pull."""
+    def pull(self, source, overwrite=False, stop_revision=None,
+        _hook_master=None, _run_hooks=True):
+        """See Branch.pull.
+
+        :param _hook_master: Private parameter - set the branch to 
+            be supplied as the master to push hooks.
+        :param _run_hooks: Private parameter - allow disabling of
+            hooks, used when pushing to a master branch.
+        """
         source.lock_read()
         try:
-            old_count = self.last_revision_info()[0]
+            old_count, old_tip = self.last_revision_info()
             try:
                 self.update_revisions(source, stop_revision)
             except DivergedBranches:
@@ -1261,17 +1296,33 @@
                     raise
             if overwrite:
                 self.set_revision_history(source.revision_history())
-            new_count = self.last_revision_info()[0]
+            new_count, new_tip = self.last_revision_info()
+            if _run_hooks:
+                if _hook_master:
+                    _hook_local = self
+                else:
+                    _hook_master = self
+                    _hook_local = None
+                for hook in Branch.hooks['post_pull']:
+                    hook(source, _hook_local, _hook_master, old_count, old_tip,
+                        new_count, new_tip)
             return new_count - old_count
         finally:
             source.unlock()
 
     @needs_read_lock
-    def push(self, target, overwrite=False, stop_revision=None):
-        """See Branch.push."""
+    def push(self, target, overwrite=False, stop_revision=None,
+        _hook_master=None, _run_hooks=True):
+        """See Branch.push.
+        
+        :param _hook_master: Private parameter - set the branch to 
+            be supplied as the master to push hooks.
+        :param _run_hooks: Private parameter - allow disabling of
+            hooks, used when pushing to a master branch.
+        """
         target.lock_write()
         try:
-            old_count = len(target.revision_history())
+            old_count, old_tip = target.last_revision_info()
             try:
                 target.update_revisions(self, stop_revision)
             except DivergedBranches:
@@ -1279,7 +1330,16 @@
                     raise
             if overwrite:
                 target.set_revision_history(self.revision_history())
-            new_count = len(target.revision_history())
+            new_count, new_tip = target.last_revision_info()
+            if _run_hooks:
+                if _hook_master:
+                    _hook_local = target
+                else:
+                    _hook_master = target
+                    _hook_local = None
+                for hook in Branch.hooks['post_push']:
+                    hook(self, _hook_local, _hook_master, old_count, old_tip,
+                        new_count, new_tip)
             return new_count - old_count
         finally:
             target.unlock()
@@ -1361,32 +1421,53 @@
                                          _repository=_repository)
         
     @needs_write_lock
-    def pull(self, source, overwrite=False, stop_revision=None):
-        """Extends branch.pull to be bound branch aware."""
+    def pull(self, source, overwrite=False, stop_revision=None,
+        _run_hooks=True):
+        """Extends branch.pull to be bound branch aware.
+        
+        :param _run_hooks: Private parameter used to force hook running
+            off during bound branch double-pushing.
+        """
         bound_location = self.get_bound_location()
-        if source.base != bound_location:
+        master_branch = None
+        if bound_location and source.base != bound_location:
             # not pulling from master, so we need to update master.
             master_branch = self.get_master_branch()
-            if master_branch:
-                master_branch.pull(source)
-                source = master_branch
-        return super(BzrBranch5, self).pull(source, overwrite, stop_revision)
+            master_branch.lock_write()
+        try:
+            if master_branch:
+                # pull from source into master.
+                master_branch.pull(source, overwrite, stop_revision,
+                    _run_hooks=False)
+            return super(BzrBranch5, self).pull(source, overwrite,
+                stop_revision, _hook_master=master_branch,
+                _run_hooks=_run_hooks)
+        finally:
+            if master_branch:
+                master_branch.unlock()
 
-    @needs_write_lock
+    @needs_read_lock
     def push(self, target, overwrite=False, stop_revision=None):
         """Updates branch.push to be bound branch aware."""
         bound_location = target.get_bound_location()
-        if target.base != bound_location:
+        master_branch = None
+        if bound_location and target.base != bound_location:
             # not pushing to master, so we need to update master.
             master_branch = target.get_master_branch()
+            master_branch.lock_write()
+        try:
             if master_branch:
                 # push into the master from this branch.
                 super(BzrBranch5, self).push(master_branch, overwrite,
-                    stop_revision)
-        # and push into the target branch from this. Note that we push from
-        # this branch again, because its considered the highest bandwidth
-        # repository.
-        return super(BzrBranch5, self).push(target, overwrite, stop_revision)
+                    stop_revision, _run_hooks=False)
+            # and push into the target branch from this. Note that we push from
+            # this branch again, because its considered the highest bandwidth
+            # repository.
+            return super(BzrBranch5, self).push(target, overwrite,
+                stop_revision, _hook_master=master_branch)
+        finally:
+            if master_branch:
+                master_branch.unlock()
 
     def get_bound_location(self):
         try:

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-02-14 21:40:53 +0000
+++ bzrlib/builtins.py	2007-02-14 21:57:40 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2006 Canonical Ltd
+# Copyright (C) 2004, 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -537,19 +537,29 @@
     location can be accessed.
     """
 
-    takes_options = ['remember', 'overwrite', 'revision', 'verbose']
+    takes_options = ['remember', 'overwrite', 'revision', 'verbose',
+        Option('directory',
+            help='branch to pull into, '
+                 'rather than the one containing the working directory',
+            short_name='d',
+            type=unicode,
+            ),
+        ]
     takes_args = ['location?']
     encoding_type = 'replace'
 
     def run(self, location=None, remember=False, overwrite=False,
-            revision=None, verbose=False):
+            revision=None, verbose=False,
+            directory=None):
         # FIXME: too much stuff is in the command class
+        if directory is None:
+            directory = u'.'
         try:
-            tree_to = WorkingTree.open_containing(u'.')[0]
+            tree_to = WorkingTree.open_containing(directory)[0]
             branch_to = tree_to.branch
         except errors.NoWorkingTree:
             tree_to = None
-            branch_to = Branch.open_containing(u'.')[0]
+            branch_to = Branch.open_containing(directory)[0]
 
         reader = None
         if location is not None:
@@ -569,7 +579,6 @@
                 self.outf.write("Using saved location: %s\n" % display_url)
                 location = stored_loc
 
-
         if reader is not None:
             install_bundle(branch_to.repository, reader)
             branch_from = branch_to
@@ -632,18 +641,33 @@
     """
 
     takes_options = ['remember', 'overwrite', 'verbose',
-                     Option('create-prefix', 
-                            help='Create the path leading up to the branch '
-                                 'if it does not already exist')]
+        Option('create-prefix',
+               help='Create the path leading up to the branch '
+                    'if it does not already exist'),
+        Option('directory',
+            help='branch to push from, '
+                 'rather than the one containing the working directory',
+            short_name='d',
+            type=unicode,
+            ),
+        Option('use-existing-dir',
+               help='By default push will fail if the target'
+                    ' directory exists, but does not already'
+                    ' have a control directory. This flag will'
+                    ' allow push to proceed.'),
+        ]
     takes_args = ['location?']
     encoding_type = 'replace'
 
     def run(self, location=None, remember=False, overwrite=False,
-            create_prefix=False, verbose=False):
+            create_prefix=False, verbose=False,
+            use_existing_dir=False,
+            directory=None):
         # FIXME: Way too big!  Put this into a function called from the
         # command.
-        
-        br_from = Branch.open_containing('.')[0]
+        if directory is None:
+            directory = '.'
+        br_from = Branch.open_containing(directory)[0]
         stored_loc = br_from.get_push_location()
         if location is None:
             if stored_loc is None:
@@ -658,35 +682,73 @@
         location_url = to_transport.base
 
         old_rh = []
+        count = 0
+
+        br_to = repository_to = dir_to = None
         try:
-            dir_to = bzrdir.BzrDir.open(location_url)
-            br_to = dir_to.open_branch()
+            dir_to = bzrdir.BzrDir.open_from_transport(to_transport)
         except errors.NotBranchError:
-            # create a branch.
-            to_transport = to_transport.clone('..')
-            if not create_prefix:
+            pass # Didn't find anything
+        else:
+            # If we can open a branch, use its direct repository, otherwise see
+            # if there is a repository without a branch.
+            try:
+                br_to = dir_to.open_branch()
+            except errors.NotBranchError:
+                # Didn't find a branch, can we find a repository?
                 try:
-                    relurl = to_transport.relpath(location_url)
-                    mutter('creating directory %s => %s', location_url, relurl)
-                    to_transport.mkdir(relurl)
-                except errors.NoSuchFile:
-                    raise errors.BzrCommandError("Parent directory of %s "
-                                                 "does not exist." % location)
+                    repository_to = dir_to.find_repository()
+                except errors.NoRepositoryPresent:
+                    pass
             else:
-                current = to_transport.base
-                needed = [(to_transport, to_transport.relpath(location_url))]
+                # Found a branch, so we must have found a repository
+                repository_to = br_to.repository
+
+        old_rh = []
+        if dir_to is None:
+            # XXX: Refactor the create_prefix/no_create_prefix code into a
+            #      common helper function
+            try:
+                to_transport.mkdir('.')
+            except errors.FileExists:
+                if not use_existing_dir:
+                    raise errors.BzrCommandError("Target directory %s"
+                         " already exists, but does not have a valid .bzr"
+                         " directory. Supply --use-existing-dir to push"
+                         " there anyway." % location)
+            except errors.NoSuchFile:
+                if not create_prefix:
+                    raise errors.BzrCommandError("Parent directory of %s"
+                        " does not exist."
+                        "\nYou may supply --create-prefix to create all"
+                        " leading parent directories."
+                        % location)
+
+                cur_transport = to_transport
+                needed = [cur_transport]
+                # Recurse upwards until we can create a directory successfully
+                while True:
+                    new_transport = cur_transport.clone('..')
+                    if new_transport.base == cur_transport.base:
+                        raise errors.BzrCommandError("Failed to create path"
+                                                     " prefix for %s."
+                                                     % location)
+                    try:
+                        new_transport.mkdir('.')
+                    except errors.NoSuchFile:
+                        needed.append(new_transport)
+                        cur_transport = new_transport
+                    else:
+                        break
+
+                # Now we only need to create child directories
                 while needed:
-                    try:
-                        to_transport, relpath = needed[-1]
-                        to_transport.mkdir(relpath)
-                        needed.pop()
-                    except errors.NoSuchFile:
-                        new_transport = to_transport.clone('..')
-                        needed.append((new_transport,
-                                       new_transport.relpath(to_transport.base)))
-                        if new_transport.base == to_transport.base:
-                            raise errors.BzrCommandError("Could not create "
-                                                         "path prefix.")
+                    cur_transport = needed.pop()
+                    cur_transport.mkdir('.')
+            
+            # Now the target directory exists, but doesn't have a .bzr
+            # directory. So we need to create it, along with any work to create
+            # all of the dependent branches, etc.
             dir_to = br_from.bzrdir.clone(location_url,
                 revision_id=br_from.last_revision())
             br_to = dir_to.open_branch()
@@ -694,7 +756,25 @@
             # We successfully created the target, remember it
             if br_from.get_push_location() is None or remember:
                 br_from.set_push_location(br_to.base)
-        else:
+        elif repository_to is None:
+            # we have a bzrdir but no branch or repository
+            # XXX: Figure out what to do other than complain.
+            raise errors.BzrCommandError("At %s you have a valid .bzr control"
+                " directory, but not a branch or repository. This is an"
+                " unsupported configuration. Please move the target directory"
+                " out of the way and try again."
+                % location)
+        elif br_to is None:
+            # We have a repository but no branch, copy the revisions, and then
+            # create a branch.
+            last_revision_id = br_from.last_revision()
+            repository_to.fetch(br_from.repository,
+                                revision_id=last_revision_id)
+            br_to = br_from.clone(dir_to, revision_id=last_revision_id)
+            count = len(br_to.revision_history())
+            if br_from.get_push_location() is None or remember:
+                br_from.set_push_location(br_to.base)
+        else: # We have a valid to branch
             # We were able to connect to the remote location, so remember it
             # we don't need to successfully push because of possible divergence.
             if br_from.get_push_location() is None or remember:
@@ -753,14 +833,8 @@
         elif len(revision) > 1:
             raise errors.BzrCommandError(
                 'bzr branch --revision takes exactly 1 revision value')
-        try:
-            br_from = Branch.open(from_location)
-        except OSError, e:
-            if e.errno == errno.ENOENT:
-                raise errors.BzrCommandError('Source location "%s" does not'
-                                             ' exist.' % to_location)
-            else:
-                raise
+
+        br_from = Branch.open(from_location)
         br_from.lock_read()
         try:
             if basis is not None:
@@ -828,6 +902,8 @@
     --basis is to speed up checking out from remote branches.  When specified, it
     uses the inventory and file contents from the basis branch in preference to the
     branch being checked out.
+
+    See "help checkouts" for more information on checkouts.
     """
     takes_args = ['branch_location?', 'to_location?']
     takes_options = ['revision', # , 'basis']
@@ -1124,13 +1200,15 @@
     """
     takes_args = ['location?']
     takes_options = [
-                     RegistryOption('format',
-                            help='Specify a format for this branch. See "bzr '
-                            'help formats" for details',
-                            converter=bzrdir.format_registry.make_bzrdir,
-                            registry=bzrdir.format_registry,
-                            value_switches=True, title="Branch Format"),
-                     ]
+         RegistryOption('format',
+                help='Specify a format for this branch. '
+                'See "help formats".',
+                registry=bzrdir.format_registry,
+                converter=bzrdir.format_registry.make_bzrdir,
+                value_switches=True,
+                title="Branch Format",
+                ),
+         ]
     def run(self, location=None, format=None):
         if format is None:
             format = bzrdir.format_registry.make_bzrdir('default')
@@ -1181,7 +1259,7 @@
         cd trunk-checkout
         (add files here)
     """
-    takes_args = ["location"] 
+    takes_args = ["location"]
     takes_options = [RegistryOption('format',
                             help='Specify a format for this repository. See'
                                  ' "bzr help formats" for details',
@@ -1447,15 +1525,12 @@
         if location:
             # find the file id to log:
 
-            dir, fp = bzrdir.BzrDir.open_containing(location)
-            b = dir.open_branch()
+            tree, b, fp = bzrdir.BzrDir.open_containing_tree_or_branch(
+                location)
             if fp != '':
-                try:
-                    # might be a tree:
-                    inv = dir.open_workingtree().inventory
-                except (errors.NotBranchError, errors.NotLocalUrl):
-                    # either no tree, or is remote.
-                    inv = b.basis_tree().inventory
+                if tree is None:
+                    tree = b.basis_tree()
+                inv = tree.inventory
                 file_id = inv.path2id(fp)
                 if file_id is None:
                     raise errors.BzrCommandError(
@@ -2296,6 +2371,10 @@
     default, use --remember. The value will only be saved if the remote
     location can be accessed.
 
+    The results of the merge are placed into the destination working
+    directory, where they can be reviewed (with bzr diff), tested, and then
+    committed to record the result of the merge.
+
     Examples:
 
     To merge the latest revision from bzr.dev
@@ -2314,27 +2393,32 @@
     """
     takes_args = ['branch?']
     takes_options = ['revision', 'force', 'merge-type', 'reprocess', 'remember',
-                     Option('show-base', help="Show base revision text in "
-                            "conflicts"),
-                     Option('uncommitted', help='Apply uncommitted changes'
-                            ' from a working copy, instead of branch changes'),
-                     Option('pull', help='If the destination is already'
-                             ' completely merged into the source, pull from the'
-                             ' source rather than merging. When this happens,'
-                             ' you do not need to commit the result.'),
-                     ]
-
-    def help(self):
-        from inspect import getdoc
-        return getdoc(self) + '\n' + _mod_merge.merge_type_help()
+        Option('show-base', help="Show base revision text in "
+               "conflicts"),
+        Option('uncommitted', help='Apply uncommitted changes'
+               ' from a working copy, instead of branch changes'),
+        Option('pull', help='If the destination is already'
+                ' completely merged into the source, pull from the'
+                ' source rather than merging. When this happens,'
+                ' you do not need to commit the result.'),
+        Option('directory',
+            help='branch to merge into, '
+                 'rather than the one containing the working directory',
+            short_name='d',
+            type=unicode,
+            ),
+    ]
 
     def run(self, branch=None, revision=None, force=False, merge_type=None,
-            show_base=False, reprocess=False, remember=False, 
-            uncommitted=False, pull=False):
+            show_base=False, reprocess=False, remember=False,
+            uncommitted=False, pull=False,
+            directory=None,
+            ):
         if merge_type is None:
             merge_type = _mod_merge.Merge3Merger
 
-        tree = WorkingTree.open_containing(u'.')[0]
+        if directory is None: directory = u'.'
+        tree = WorkingTree.open_containing(directory)[0]
         change_reporter = delta.ChangeReporter(tree.inventory)
 
         if branch is not None:
@@ -2403,8 +2487,8 @@
                     reprocess=reprocess,
                     show_base=show_base,
                     pull=pull,
-                    pb=pb,
-                    file_list=interesting_files,
+                    this_dir=directory,
+                    pb=pb, file_list=interesting_files,
                     change_reporter=change_reporter)
             finally:
                 pb.finished()
@@ -2464,10 +2548,6 @@
                      Option('show-base', help="Show base revision text in "
                             "conflicts")]
 
-    def help(self):
-        from inspect import getdoc
-        return getdoc(self) + '\n' + _mod_merge.merge_type_help()
-
     def run(self, file_list=None, merge_type=None, show_base=False,
             reprocess=False):
         if merge_type is None:
@@ -2855,10 +2935,12 @@
 
 
 class cmd_bind(Command):
-    """Bind the current branch to a master branch.
-
-    After binding, commits must succeed on the master branch
-    before they are executed on the local one.
+    """Convert the current branch into a checkout of the supplied branch.
+
+    Once converted into a checkout, commits must succeed on the master branch
+    before they will be applied to the local branch.
+
+    See "help checkouts" for more information on checkouts.
     """
 
     takes_args = ['location']
@@ -2875,10 +2957,12 @@
 
 
 class cmd_unbind(Command):
-    """Unbind the current branch from its master branch.
-
-    After unbinding, the local branch is considered independent.
-    All subsequent commits will be local.
+    """Convert the current checkout into a regular branch.
+
+    After unbinding, the local branch is considered independent and subsequent
+    commits will be local only.
+
+    See "help checkouts" for more information on checkouts.
     """
 
     takes_args = []

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2007-01-26 21:56:16 +0000
+++ bzrlib/bzrdir.py	2007-02-14 07:10:27 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -45,7 +45,6 @@
     lockdir,
     registry,
     revision as _mod_revision,
-    repository as _mod_repository,
     symbol_versioning,
     urlutils,
     xml4,
@@ -873,7 +872,7 @@
 
     def open_repository(self):
         """See BzrDir.open_repository."""
-        from bzrlib.repository import RepositoryFormat4
+        from bzrlib.repofmt.weaverepo import RepositoryFormat4
         return RepositoryFormat4().open(self, _found=True)
 
 
@@ -885,7 +884,7 @@
 
     def open_repository(self):
         """See BzrDir.open_repository."""
-        from bzrlib.repository import RepositoryFormat5
+        from bzrlib.repofmt.weaverepo import RepositoryFormat5
         return RepositoryFormat5().open(self, _found=True)
 
     def open_workingtree(self, _unsupported=False):
@@ -902,7 +901,7 @@
 
     def open_repository(self):
         """See BzrDir.open_repository."""
-        from bzrlib.repository import RepositoryFormat6
+        from bzrlib.repofmt.weaverepo import RepositoryFormat6
         return RepositoryFormat6().open(self, _found=True)
 
     def open_workingtree(self, _unsupported=False):
@@ -1303,7 +1302,7 @@
 
     def __return_repository_format(self):
         """Circular import protection."""
-        from bzrlib.repository import RepositoryFormat4
+        from bzrlib.repofmt.weaverepo import RepositoryFormat4
         return RepositoryFormat4()
     repository_format = property(__return_repository_format)
 
@@ -1343,7 +1342,7 @@
         Except when they are being cloned.
         """
         from bzrlib.branch import BzrBranchFormat4
-        from bzrlib.repository import RepositoryFormat5
+        from bzrlib.repofmt.weaverepo import RepositoryFormat5
         from bzrlib.workingtree import WorkingTreeFormat2
         result = (super(BzrDirFormat5, self).initialize_on_transport(transport))
         RepositoryFormat5().initialize(result, _internal=True)
@@ -1363,7 +1362,7 @@
 
     def __return_repository_format(self):
         """Circular import protection."""
-        from bzrlib.repository import RepositoryFormat5
+        from bzrlib.repofmt.weaverepo import RepositoryFormat5
         return RepositoryFormat5()
     repository_format = property(__return_repository_format)
 
@@ -1402,7 +1401,7 @@
         Except when they are being cloned.
         """
         from bzrlib.branch import BzrBranchFormat4
-        from bzrlib.repository import RepositoryFormat6
+        from bzrlib.repofmt.weaverepo import RepositoryFormat6
         from bzrlib.workingtree import WorkingTreeFormat2
         result = super(BzrDirFormat6, self).initialize_on_transport(transport)
         RepositoryFormat6().initialize(result, _internal=True)
@@ -1422,7 +1421,7 @@
 
     def __return_repository_format(self):
         """Circular import protection."""
-        from bzrlib.repository import RepositoryFormat6
+        from bzrlib.repofmt.weaverepo import RepositoryFormat6
         return RepositoryFormat6()
     repository_format = property(__return_repository_format)
 
@@ -1834,6 +1833,7 @@
 
     def convert(self, to_convert, pb):
         """See Converter.convert()."""
+        from bzrlib.repofmt.weaverepo import RepositoryFormat7
         from bzrlib.branch import BzrBranchFormat5
         self.bzrdir = to_convert
         self.pb = pb
@@ -1869,7 +1869,7 @@
         # we hard code the formats here because we are converting into
         # the meta format. The meta format upgrader can take this to a 
         # future format within each component.
-        self.put_format('repository', _mod_repository.RepositoryFormat7())
+        self.put_format('repository', RepositoryFormat7())
         for entry in repository_names:
             self.move_entry('repository', entry)
 
@@ -1989,16 +1989,30 @@
 
     def register_metadir(self, key, repo, help, native=True, deprecated=False):
         """Register a metadir subformat.
-        
-        repo is the repository format name as a string.
+
+        These all use a BzrDirMetaFormat1 bzrdir, but can be parameterized
+        by the Repository format.
+
+        :param repo: The fully-qualified repository format class name as a
+        string.
         """
         # This should be expanded to support setting WorkingTree and Branch
         # formats, once BzrDirMetaFormat1 supports that.
         def helper():
-            import bzrlib.repository
-            repo_format = getattr(bzrlib.repository, repo)
+            mod_name, repo_factory_name = repo.rsplit('.', 1)
+            try:
+                mod = __import__(mod_name, globals(), locals(),
+                        [repo_factory_name])
+            except ImportError, e:
+                raise ImportError('failed to load repository %s: %s'
+                    % (repo, e))
+            try:
+                repo_format_class = getattr(mod, repo_factory_name)
+            except AttributeError:
+                raise AttributeError('no repository format %r in module %r' 
+                    % (repo, mod))
             bd = BzrDirMetaFormat1()
-            bd.repository_format = repo_format()
+            bd.repository_format = repo_format_class()
             return bd
         self.register(key, helper, help, native, deprecated)
 
@@ -2089,12 +2103,19 @@
 format_registry = BzrDirFormatRegistry()
 format_registry.register('weave', BzrDirFormat6,
     'Pre-0.8 format.  Slower than knit and does not'
-    ' support checkouts or shared repositories.', deprecated=True)
-format_registry.register_metadir('knit', 'RepositoryFormatKnit1',
-    'Format using knits.  Recommended.')
+    ' support checkouts or shared repositories.',
+    deprecated=True)
+format_registry.register_metadir('knit',
+    'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
+    'Format using knits.  Recommended.',
+    )
 format_registry.set_default('knit')
-format_registry.register_metadir('metaweave', 'RepositoryFormat7',
+format_registry.register_metadir('metaweave',
+    'bzrlib.repofmt.weaverepo.RepositoryFormat7',
     'Transitional format in 0.8.  Slower than knit.',
-    deprecated=True)
-format_registry.register_metadir('experimental-knit2', 'RepositoryFormatKnit2',
-    'Experimental successor to knit.  Use at your own risk.')
+    deprecated=True,
+    )
+format_registry.register_metadir('experimental-knit2',
+    'bzrlib.repofmt.knitrepo.RepositoryFormatKnit2',
+    'Experimental successor to knit.  Use at your own risk.',
+    )

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2007-02-06 14:54:06 +0000
+++ bzrlib/commit.py	2007-02-06 23:53:30 +0000
@@ -61,6 +61,7 @@
     inventory,
     tree,
     )
+from bzrlib.branch import Branch
 import bzrlib.config
 from bzrlib.errors import (BzrError, PointlessCommit,
                            ConflictsInTree,
@@ -332,6 +333,8 @@
             # now the work tree is up to date with the branch
             
             self.reporter.completed(self.branch.revno(), self.rev_id)
+            # old style commit hooks - should be deprecated ? (obsoleted in
+            # 0.15)
             if self.config.post_commit() is not None:
                 hooks = self.config.post_commit().split(' ')
                 # this would be nicer with twisted.python.reflect.namedAny
@@ -340,6 +343,25 @@
                                   {'branch':self.branch,
                                    'bzrlib':bzrlib,
                                    'rev_id':self.rev_id})
+            # new style commit hooks:
+            if not self.bound_branch:
+                hook_master = self.branch
+                hook_local = None
+            else:
+                hook_master = self.master_branch
+                hook_local = self.branch
+            new_revno = self.branch.revno()
+            # With bound branches, when the master is behind the local branch,
+            # the 'old_revno' and old_revid values here are incorrect.
+            # XXX: FIXME ^. RBC 20060206
+            old_revno = new_revno - 1
+            if self.parents:
+                old_revid = self.parents[0]
+            else:
+                old_revid = bzrlib.revision.NULL_REVISION
+            for hook in Branch.hooks['post_commit']:
+                hook(hook_local, hook_master, old_revno, old_revid, new_revno,
+                    self.rev_id)
             self._emit_progress_update()
         finally:
             self._cleanup()

=== modified file 'bzrlib/gpg.py'
--- bzrlib/gpg.py	2006-10-16 01:25:46 +0000
+++ bzrlib/gpg.py	2007-02-08 16:28:05 +0000
@@ -76,6 +76,8 @@
         self._config = config
 
     def sign(self, content):
+        if isinstance(content, unicode):
+            raise errors.BzrBadParameterUnicode('content')
         ui.ui_factory.clear_term()
 
         preexec_fn = _set_gpg_tty

=== modified file 'bzrlib/help_topics.py'
--- bzrlib/help_topics.py	2007-01-31 16:33:48 +0000
+++ bzrlib/help_topics.py	2007-02-07 22:31:07 +0000
@@ -151,6 +151,84 @@
 Note: --version must be supplied before any command.
 """
 
+_checkouts = \
+"""Checkouts
+
+Checkouts are source trees that are connected to a branch, so that when
+you commit in the source tree, the commit goes into that branch.  They
+allow you to use a simpler, more centralized workflow, ignoring some of
+Bazaar's decentralized features until you want them. Using checkouts
+with shared repositories is very similar to working with SVN or CVS, but
+doesn't have the same restrictions.  And using checkouts still allows
+others working on the project to use whatever workflow they like.
+
+A checkout is created with the bzr checkout command (see "help checkout").
+You pass it a reference to another branch, and it will create a local copy
+for you that still contains a reference to the branch you created the
+checkout from (the master branch). Then if you make any commits they will be
+made on the other branch first. This creates an instant mirror of your work, or
+facilitates lockstep development, where each developer is working together,
+continuously integrating the changes of others.
+
+However the checkout is still a first class branch in Bazaar terms, so that
+you have the full history locally.  As you have a first class branch you can
+also commit locally if you want, for instance due to the temporary loss af a
+network connection. Use the --local option to commit to do this. All the local
+commits will then be made on the master branch the next time you do a non-local
+commit.
+
+If you are using a checkout from a shared branch you will periodically want to
+pull in all the changes made by others. This is done using the "update"
+command. The changes need to be applied before any non-local commit, but
+Bazaar will tell you if there are any changes and suggest that you use this
+command when needed.
+
+It is also possible to create a "lightweight" checkout by passing the
+--lightweight flag to checkout. A lightweight checkout is even closer to an
+SVN checkout in that it is not a first class branch, it mainly consists of the
+working tree. This means that any history operations must query the master
+branch, which could be slow if a network connection is involved. Also, as you
+don't have a local branch, then you cannot commit locally.
+
+Lightwieght checkouts work best when you have fast reliable access to the
+master branch. This means that if the master branch is on the same disk or LAN
+a lightweight checkout will be faster than a heavyweight one for any commands
+that modify the revision history (as only one copy branch needs to be updated).
+Heavyweight checkouts will generally be faster for any command that uses the
+history but does not change it, but if the master branch is on the same disk
+then there wont be a noticeable difference.
+
+Another possible use for a checkout is to use it with a treeless repository
+containing your branches, where you maintain only only one working tree by
+switching the master branch that the checkout points to when you want to 
+work on a different branch.
+
+Obviously to commit on a checkout you need to be able to write to the master
+branch. This means that the master branch must be accessable over a writeable
+protocol , such as sftp://, and that you have write permissions at the other
+end. Checkouts also work on the local file system, so that all that matters is
+file permissions.
+
+You can change the master of a checkout by using the "bind" command (see "help
+bind"). This will change the location that the commits are sent to. The bind
+command can also be used to turn a branch into a heavy checkout. If you
+would like to convert your heavy checkout into a normal branch so that every
+commit is local, you can use the "unbind" command.
+
+Related commands:
+
+  checkout    Create a checkout. Pass --lightweight to get a lightweight
+              checkout
+  update      Pull any changes in the master branch in to your checkout
+  commit      Make a commit that is sent to the master branch. If you have
+              a heavy checkout then the --local option will commit to the 
+              checkout without sending the commit to the master
+  bind        Change the master branch that the commits in the checkout will
+              be sent to
+  unbind      Turn a heavy checkout into a standalone branch so that any
+              commits are only made locally
+"""
+
 
 topic_registry.register("revisionspec", _help_on_revisionspec,
                         "Explain how to use --revision")
@@ -162,3 +240,6 @@
 topic_registry.register('formats', get_format_topic, 'Directory formats')
 topic_registry.register('global-options', _global_options,
                         'Options that can be used with any command')
+topic_registry.register('checkouts', _checkouts,
+                        'Information on what a checkout is')
+

=== modified file 'bzrlib/inter.py'
--- bzrlib/inter.py	2006-08-22 16:14:17 +0000
+++ bzrlib/inter.py	2007-02-06 06:27:24 +0000
@@ -34,6 +34,10 @@
     lock_read, lock_write, unlock, then the InterObject's lock_read,
     lock_write and unlock methods may be used (optionally in conjunction with
     the needs_read_lock and needs_write_lock decorators.)
+
+    When looking for an inter, the most recently registered types are tested
+    first.  So typically the most generic and slowest InterObjects should be
+    registered first.
     """
 
     # _optimisers = list()

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2007-02-14 21:40:53 +0000
+++ bzrlib/merge.py	2007-02-14 21:57:40 +0000
@@ -21,6 +21,7 @@
 
 from bzrlib import (
     osutils,
+    registry,
     )
 from bzrlib.branch import Branch
 from bzrlib.conflicts import ConflictList, Conflict
@@ -969,14 +970,10 @@
     merger.other_basis = other_rev_id
     return merger.do_merge()
 
-
-merge_types = {     "merge3": (Merge3Merger, "Native diff3-style merge"), 
-                     "diff3": (Diff3Merger,  "Merge using external diff3"),
-                     'weave': (WeaveMerger, "Weave-based merge")
-              }
-
-
-def merge_type_help():
-    templ = '%s%%7s: %%s' % (' '*12)
-    lines = [templ % (f[0], f[1][1]) for f in merge_types.iteritems()]
-    return '\n'.join(lines)
+def get_merge_type_registry():
+    """Merge type registry is in bzrlib.option to avoid circular imports.
+
+    This method provides a sanctioned way to retrieve it.
+    """
+    from bzrlib import option
+    return option._merge_type_registry

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2007-02-07 13:59:03 +0000
+++ bzrlib/option.py	2007-02-13 05:22:39 +0000
@@ -26,6 +26,7 @@
 from bzrlib import (
     errors,
     log,
+    registry,
     revisionspec,
     symbol_versioning,
     )
@@ -236,8 +237,8 @@
         :param converter: Callable to invoke with the value name to produce
             the value.  If not supplied, self.registry.get is used.
         :param value_switches: If true, each possible value is assigned its
-            own switch.  For example, instead of '--format metaweave',
-            '--metaweave' can be used interchangeably.
+            own switch.  For example, instead of '--format knit',
+            '--knit' can be used interchangeably.
         """
         Option.__init__(self, name, help, type=self.convert)
         self.registry = registry
@@ -306,6 +307,19 @@
     Option.OPTIONS[name] = RegistryOption(name, help, registry, **kwargs)
 
 
+class MergeTypeRegistry(registry.Registry):
+
+    pass
+
+
+_merge_type_registry = MergeTypeRegistry()
+_merge_type_registry.register_lazy('merge3', 'bzrlib.merge', 'Merge3Merger',
+                                   "Native diff3-style merge")
+_merge_type_registry.register_lazy('diff3', 'bzrlib.merge', 'Diff3Merger',
+                                   "Merge using external diff3")
+_merge_type_registry.register_lazy('weave', 'bzrlib.merge', 'WeaveMerger',
+                                   "Weave-based merge")
+
 _global_option('all')
 _global_option('overwrite', help='Ignore differences between branches and '
                'overwrite unconditionally')
@@ -349,8 +363,9 @@
 _global_option('line', help='Use log format with one line per revision. Same as --log-format line')
 _global_option('root', type=str)
 _global_option('no-backup')
-_global_option('merge-type', type=_parse_merge_type, 
-               help='Select a particular merge algorithm')
+_global_registry_option('merge-type', 'Select a particular merge algorithm',
+                        _merge_type_registry, value_switches=True,
+                        title='Merge algorithm')
 _global_option('pattern', type=str)
 _global_option('quiet', short_name='q')
 _global_option('remember', help='Remember the specified location as a'

=== modified file 'bzrlib/plugins/launchpad/__init__.py'
--- bzrlib/plugins/launchpad/__init__.py	2006-10-11 23:08:27 +0000
+++ bzrlib/plugins/launchpad/__init__.py	2007-01-26 04:44:10 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,9 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Launchpad.net branch registration plugin for bzr
-
-This adds commands that tell launchpad about newly-created branches, etc.
+"""Launchpad.net integration plugin for Bazaar
 
 To install this file, put the 'bzr_lp' directory, or a symlink to it,
 in your ~/.bazaar/plugins/ directory.
@@ -28,7 +26,8 @@
 # see http://bazaar-vcs.org/Specs/BranchRegistrationTool
 
 from bzrlib.commands import Command, Option, register_command
-
+from bzrlib.transport import register_lazy_transport
+from bzrlib.help_topics import topic_registry
 
 
 class cmd_register_branch(Command):
@@ -115,8 +114,48 @@
 
 register_command(cmd_register_branch)
 
+register_lazy_transport(
+    'lp:',
+    'bzrlib.plugins.launchpad.lp_indirect',
+    'launchpad_transport_indirect')
+
+register_lazy_transport(
+    'lp://',
+    'bzrlib.plugins.launchpad.lp_indirect',
+    'launchpad_transport_indirect')
+
 def test_suite():
     """Called by bzrlib to fetch tests for this plugin"""
     from unittest import TestSuite, TestLoader
     import test_register
-    return TestLoader().loadTestsFromModule(test_register)
+    import test_lp_indirect
+
+    loader = TestLoader()
+    suite = TestSuite()
+    for m in [test_register, test_lp_indirect]:
+        suite.addTests(loader.loadTestsFromModule(m))
+    return suite
+
+_launchpad_help = """Integration with Launchpad.net
+
+Launchpad.net provides free Bazaar branch hosting with integrated bug and
+specification tracking.
+
+The bzr client (through the plugin called 'launchpad') has two special
+features to communicate with Launchpad:
+
+    * The register-branch command tells launchpad about the url of a 
+      public branch.  Launchpad will then mirror the branch, display
+      its contents and allow it to be attached to bugs and other 
+      objects.
+
+    * The 'lp:' transport uses Launchpad as a directory service: 
+      for example 'lp:bzr' and 'lp:python' refer to the main branches of the
+      relevant projects and may be branched, logged, etc.  (Only read access
+      is supported at present.)
+
+For more information see http://help.launchpad.net/
+"""
+topic_registry.register('launchpad',
+    _launchpad_help,
+    'Using Bazaar with Launchpad.net')

=== added file 'bzrlib/plugins/launchpad/lp_indirect.py'
--- bzrlib/plugins/launchpad/lp_indirect.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/launchpad/lp_indirect.py	2007-01-26 04:44:10 +0000
@@ -0,0 +1,48 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Transport indirection that uses Launchpad as a directory lookup.
+
+When the transport is opened, it immediately redirects to a url
+on Launchpad, which can then either serve the branch itself or redirect
+again.
+"""
+
+from bzrlib import (
+    errors,
+    )
+from bzrlib.transport import (
+    get_transport,
+    Transport,
+    )
+
+
+def launchpad_transport_indirect(base_url):
+    """Uses Launchpad.net as a directory of open source software"""
+    if base_url.startswith('lp:///'):
+        real_url = 'http://code.launchpad.net/' + base_url[6:]
+    elif base_url.startswith('lp:') and base_url[3] != '/':
+        real_url = 'http://code.launchpad.net/' + base_url[3:]
+    else:
+        raise errors.InvalidURL(path=base_url)
+    return get_transport(real_url)
+
+
+def get_test_permutations():
+    # Since this transport doesn't do anything once opened, it's not subjected
+    # to the usual transport tests.
+    return []

=== added file 'bzrlib/plugins/launchpad/test_lp_indirect.py'
--- bzrlib/plugins/launchpad/test_lp_indirect.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/launchpad/test_lp_indirect.py	2007-02-08 08:33:26 +0000
@@ -0,0 +1,50 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for indirect branch urls through Launchpad.net"""
+
+from bzrlib import (
+    errors,
+    transport,
+    )
+from bzrlib.transport import get_transport
+from bzrlib.tests import TestCase, TestSkipped
+
+
+class IndirectUrlTests(TestCase):
+    """Tests for indirect branch urls through Launchpad.net"""
+
+    def test_short_form(self):
+        """A launchpad url should map to a http url"""
+        url = 'lp:apt'
+        t = get_transport(url)
+        self.assertEquals(t.base, 'http://code.launchpad.net/apt/')
+
+    def test_indirect_through_url(self):
+        """A launchpad url should map to a http url"""
+        # These can change to use the smartserver protocol or something 
+        # else in the future.
+        url = 'lp:///apt'
+        t = get_transport(url)
+        real_url = t.base
+        self.assertEquals(real_url, 'http://code.launchpad.net/apt/')
+
+    # TODO: check we get an error if the url is unreasonable
+    def test_error_for_bad_indirection(self):
+        self.assertRaises(errors.InvalidURL,
+            get_transport,
+            'lp://ratotehunoahu')
+

=== added file 'bzrlib/repofmt/__init__.py'
--- bzrlib/repofmt/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/repofmt/__init__.py	2007-02-06 06:27:24 +0000
@@ -0,0 +1,17 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Repository formats"""

=== added file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	1970-01-01 00:00:00 +0000
+++ bzrlib/repofmt/knitrepo.py	2007-02-07 12:41:45 +0000
@@ -0,0 +1,430 @@
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib import (
+    errors,
+    graph,
+    knit,
+    lockable_files,
+    lockdir,
+    transactions,
+    xml5,
+    xml6,
+    )
+
+from bzrlib.decorators import needs_read_lock, needs_write_lock
+from bzrlib.repository import (
+    MetaDirRepository,
+    MetaDirRepositoryFormat,
+    RepositoryFormat,
+    RootCommitBuilder,
+    )
+import bzrlib.revision as _mod_revision
+from bzrlib.store.versioned import VersionedFileStore
+from bzrlib.trace import mutter, note, warning
+
+
+class KnitRepository(MetaDirRepository):
+    """Knit format repository."""
+
+
+    _serializer = xml5.serializer_v5
+
+    def _warn_if_deprecated(self):
+        # This class isn't deprecated
+        pass
+
+    def _inventory_add_lines(self, inv_vf, revid, parents, lines):
+        inv_vf.add_lines_with_ghosts(revid, parents, lines)
+
+    @needs_read_lock
+    def _all_revision_ids(self):
+        """See Repository.all_revision_ids()."""
+        # Knits get the revision graph from the index of the revision knit, so
+        # it's always possible even if they're on an unlistable transport.
+        return self._revision_store.all_revision_ids(self.get_transaction())
+
+    def fileid_involved_between_revs(self, from_revid, to_revid):
+        """Find file_id(s) which are involved in the changes between revisions.
+
+        This determines the set of revisions which are involved, and then
+        finds all file ids affected by those revisions.
+        """
+        vf = self._get_revision_vf()
+        from_set = set(vf.get_ancestry(from_revid))
+        to_set = set(vf.get_ancestry(to_revid))
+        changed = to_set.difference(from_set)
+        return self._fileid_involved_by_set(changed)
+
+    def fileid_involved(self, last_revid=None):
+        """Find all file_ids modified in the ancestry of last_revid.
+
+        :param last_revid: If None, last_revision() will be used.
+        """
+        if not last_revid:
+            changed = set(self.all_revision_ids())
+        else:
+            changed = set(self.get_ancestry(last_revid))
+        if None in changed:
+            changed.remove(None)
+        return self._fileid_involved_by_set(changed)
+
+    @needs_read_lock
+    def get_ancestry(self, revision_id):
+        """Return a list of revision-ids integrated by a revision.
+        
+        This is topologically sorted.
+        """
+        if revision_id is None:
+            return [None]
+        vf = self._get_revision_vf()
+        try:
+            return [None] + vf.get_ancestry(revision_id)
+        except errors.RevisionNotPresent:
+            raise errors.NoSuchRevision(self, revision_id)
+
+    @needs_read_lock
+    def get_revision(self, revision_id):
+        """Return the Revision object for a named revision"""
+        return self.get_revision_reconcile(revision_id)
+
+    @needs_read_lock
+    def get_revision_graph(self, revision_id=None):
+        """Return a dictionary containing the revision graph.
+
+        :param revision_id: The revision_id to get a graph from. If None, then
+        the entire revision graph is returned. This is a deprecated mode of
+        operation and will be removed in the future.
+        :return: a dictionary of revision_id->revision_parents_list.
+        """
+        # special case NULL_REVISION
+        if revision_id == _mod_revision.NULL_REVISION:
+            return {}
+        a_weave = self._get_revision_vf()
+        entire_graph = a_weave.get_graph()
+        if revision_id is None:
+            return a_weave.get_graph()
+        elif revision_id not in a_weave:
+            raise errors.NoSuchRevision(self, revision_id)
+        else:
+            # add what can be reached from revision_id
+            result = {}
+            pending = set([revision_id])
+            while len(pending) > 0:
+                node = pending.pop()
+                result[node] = a_weave.get_parents(node)
+                for revision_id in result[node]:
+                    if revision_id not in result:
+                        pending.add(revision_id)
+            return result
+
+    @needs_read_lock
+    def get_revision_graph_with_ghosts(self, revision_ids=None):
+        """Return a graph of the revisions with ghosts marked as applicable.
+
+        :param revision_ids: an iterable of revisions to graph or None for all.
+        :return: a Graph object with the graph reachable from revision_ids.
+        """
+        result = graph.Graph()
+        vf = self._get_revision_vf()
+        versions = set(vf.versions())
+        if not revision_ids:
+            pending = set(self.all_revision_ids())
+            required = set([])
+        else:
+            pending = set(revision_ids)
+            # special case NULL_REVISION
+            if _mod_revision.NULL_REVISION in pending:
+                pending.remove(_mod_revision.NULL_REVISION)
+            required = set(pending)
+        done = set([])
+        while len(pending):
+            revision_id = pending.pop()
+            if not revision_id in versions:
+                if revision_id in required:
+                    raise errors.NoSuchRevision(self, revision_id)
+                # a ghost
+                result.add_ghost(revision_id)
+                # mark it as done so we don't try for it again.
+                done.add(revision_id)
+                continue
+            parent_ids = vf.get_parents_with_ghosts(revision_id)
+            for parent_id in parent_ids:
+                # is this queued or done ?
+                if (parent_id not in pending and
+                    parent_id not in done):
+                    # no, queue it.
+                    pending.add(parent_id)
+            result.add_node(revision_id, parent_ids)
+            done.add(revision_id)
+        return result
+
+    def _get_revision_vf(self):
+        """:return: a versioned file containing the revisions."""
+        vf = self._revision_store.get_revision_file(self.get_transaction())
+        return vf
+
+    @needs_write_lock
+    def reconcile(self, other=None, thorough=False):
+        """Reconcile this repository."""
+        from bzrlib.reconcile import KnitReconciler
+        reconciler = KnitReconciler(self, thorough=thorough)
+        reconciler.reconcile()
+        return reconciler
+    
+    def revision_parents(self, revision_id):
+        return self._get_revision_vf().get_parents(revision_id)
+
+
+class KnitRepository2(KnitRepository):
+    """"""
+    def __init__(self, _format, a_bzrdir, control_files, _revision_store,
+                 control_store, text_store):
+        KnitRepository.__init__(self, _format, a_bzrdir, control_files,
+                              _revision_store, control_store, text_store)
+        self._serializer = xml6.serializer_v6
+
+    def deserialise_inventory(self, revision_id, xml):
+        """Transform the xml into an inventory object. 
+
+        :param revision_id: The expected revision id of the inventory.
+        :param xml: A serialised inventory.
+        """
+        result = self._serializer.read_inventory_from_string(xml)
+        assert result.root.revision is not None
+        return result
+
+    def serialise_inventory(self, inv):
+        """Transform the inventory object into XML text.
+
+        :param revision_id: The expected revision id of the inventory.
+        :param xml: A serialised inventory.
+        """
+        assert inv.revision_id is not None
+        assert inv.root.revision is not None
+        return KnitRepository.serialise_inventory(self, inv)
+
+    def get_commit_builder(self, branch, parents, config, timestamp=None,
+                           timezone=None, committer=None, revprops=None,
+                           revision_id=None):
+        """Obtain a CommitBuilder for this repository.
+        
+        :param branch: Branch to commit to.
+        :param parents: Revision ids of the parents of the new revision.
+        :param config: Configuration to use.
+        :param timestamp: Optional timestamp recorded for commit.
+        :param timezone: Optional timezone for timestamp.
+        :param committer: Optional committer to set for commit.
+        :param revprops: Optional dictionary of revision properties.
+        :param revision_id: Optional revision id.
+        """
+        return RootCommitBuilder(self, parents, config, timestamp, timezone,
+                                 committer, revprops, revision_id)
+
+
+class RepositoryFormatKnit(MetaDirRepositoryFormat):
+    """Bzr repository knit format (generalized). 
+
+    This repository format has:
+     - knits for file texts and inventory
+     - hash subdirectory based stores.
+     - knits for revisions and signatures
+     - TextStores for revisions and signatures.
+     - a format marker of its own
+     - an optional 'shared-storage' flag
+     - an optional 'no-working-trees' flag
+     - a LockDir lock
+    """
+
+    def _get_control_store(self, repo_transport, control_files):
+        """Return the control store for this repository."""
+        return VersionedFileStore(
+            repo_transport,
+            prefixed=False,
+            file_mode=control_files._file_mode,
+            versionedfile_class=knit.KnitVersionedFile,
+            versionedfile_kwargs={'factory':knit.KnitPlainFactory()},
+            )
+
+    def _get_revision_store(self, repo_transport, control_files):
+        """See RepositoryFormat._get_revision_store()."""
+        from bzrlib.store.revision.knit import KnitRevisionStore
+        versioned_file_store = VersionedFileStore(
+            repo_transport,
+            file_mode=control_files._file_mode,
+            prefixed=False,
+            precious=True,
+            versionedfile_class=knit.KnitVersionedFile,
+            versionedfile_kwargs={'delta':False,
+                                  'factory':knit.KnitPlainFactory(),
+                                 },
+            escaped=True,
+            )
+        return KnitRevisionStore(versioned_file_store)
+
+    def _get_text_store(self, transport, control_files):
+        """See RepositoryFormat._get_text_store()."""
+        return self._get_versioned_file_store('knits',
+                                  transport,
+                                  control_files,
+                                  versionedfile_class=knit.KnitVersionedFile,
+                                  versionedfile_kwargs={
+                                      'create_parent_dir':True,
+                                      'delay_create':True,
+                                      'dir_mode':control_files._dir_mode,
+                                  },
+                                  escaped=True)
+
+    def initialize(self, a_bzrdir, shared=False):
+        """Create a knit format 1 repository.
+
+        :param a_bzrdir: bzrdir to contain the new repository; must already
+            be initialized.
+        :param shared: If true the repository will be initialized as a shared
+                       repository.
+        """
+        mutter('creating repository in %s.', a_bzrdir.transport.base)
+        dirs = ['revision-store', 'knits']
+        files = []
+        utf8_files = [('format', self.get_format_string())]
+        
+        self._upload_blank_content(a_bzrdir, dirs, files, utf8_files, shared)
+        repo_transport = a_bzrdir.get_repository_transport(None)
+        control_files = lockable_files.LockableFiles(repo_transport,
+                                'lock', lockdir.LockDir)
+        control_store = self._get_control_store(repo_transport, control_files)
+        transaction = transactions.WriteTransaction()
+        # trigger a write of the inventory store.
+        control_store.get_weave_or_empty('inventory', transaction)
+        _revision_store = self._get_revision_store(repo_transport, control_files)
+        # the revision id here is irrelevant: it will not be stored, and cannot
+        # already exist.
+        _revision_store.has_revision_id('A', transaction)
+        _revision_store.get_signature_file(transaction)
+        return self.open(a_bzrdir=a_bzrdir, _found=True)
+
+    def open(self, a_bzrdir, _found=False, _override_transport=None):
+        """See RepositoryFormat.open().
+        
+        :param _override_transport: INTERNAL USE ONLY. Allows opening the
+                                    repository at a slightly different url
+                                    than normal. I.e. during 'upgrade'.
+        """
+        if not _found:
+            format = RepositoryFormat.find_format(a_bzrdir)
+            assert format.__class__ ==  self.__class__
+        if _override_transport is not None:
+            repo_transport = _override_transport
+        else:
+            repo_transport = a_bzrdir.get_repository_transport(None)
+        control_files = lockable_files.LockableFiles(repo_transport,
+                                'lock', lockdir.LockDir)
+        text_store = self._get_text_store(repo_transport, control_files)
+        control_store = self._get_control_store(repo_transport, control_files)
+        _revision_store = self._get_revision_store(repo_transport, control_files)
+        return KnitRepository(_format=self,
+                              a_bzrdir=a_bzrdir,
+                              control_files=control_files,
+                              _revision_store=_revision_store,
+                              control_store=control_store,
+                              text_store=text_store)
+
+
+class RepositoryFormatKnit1(RepositoryFormatKnit):
+    """Bzr repository knit format 1.
+
+    This repository format has:
+     - knits for file texts and inventory
+     - hash subdirectory based stores.
+     - knits for revisions and signatures
+     - TextStores for revisions and signatures.
+     - a format marker of its own
+     - an optional 'shared-storage' flag
+     - an optional 'no-working-trees' flag
+     - a LockDir lock
+
+    This format was introduced in bzr 0.8.
+    """
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar-NG Knit Repository Format 1"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Knit repository format 1"
+
+    def check_conversion_target(self, target_format):
+        pass
+
+
+class RepositoryFormatKnit2(RepositoryFormatKnit):
+    """Bzr repository knit format 2.
+
+    THIS FORMAT IS EXPERIMENTAL
+    This repository format has:
+     - knits for file texts and inventory
+     - hash subdirectory based stores.
+     - knits for revisions and signatures
+     - TextStores for revisions and signatures.
+     - a format marker of its own
+     - an optional 'shared-storage' flag
+     - an optional 'no-working-trees' flag
+     - a LockDir lock
+     - Support for recording full info about the tree root
+
+    """
+    
+    rich_root_data = True
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar Knit Repository Format 2\n"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Knit repository format 2"
+
+    def check_conversion_target(self, target_format):
+        if not target_format.rich_root_data:
+            raise errors.BadConversionTarget(
+                'Does not support rich root data.', target_format)
+
+    def open(self, a_bzrdir, _found=False, _override_transport=None):
+        """See RepositoryFormat.open().
+        
+        :param _override_transport: INTERNAL USE ONLY. Allows opening the
+                                    repository at a slightly different url
+                                    than normal. I.e. during 'upgrade'.
+        """
+        if not _found:
+            format = RepositoryFormat.find_format(a_bzrdir)
+            assert format.__class__ ==  self.__class__
+        if _override_transport is not None:
+            repo_transport = _override_transport
+        else:
+            repo_transport = a_bzrdir.get_repository_transport(None)
+        control_files = lockable_files.LockableFiles(repo_transport, 'lock',
+                                                     lockdir.LockDir)
+        text_store = self._get_text_store(repo_transport, control_files)
+        control_store = self._get_control_store(repo_transport, control_files)
+        _revision_store = self._get_revision_store(repo_transport, control_files)
+        return KnitRepository2(_format=self,
+                               a_bzrdir=a_bzrdir,
+                               control_files=control_files,
+                               _revision_store=_revision_store,
+                               control_store=control_store,
+                               text_store=text_store)

=== added file 'bzrlib/repofmt/weaverepo.py'
--- bzrlib/repofmt/weaverepo.py	1970-01-01 00:00:00 +0000
+++ bzrlib/repofmt/weaverepo.py	2007-02-12 06:35:46 +0000
@@ -0,0 +1,406 @@
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Old weave-based repository formats"""
+
+from StringIO import StringIO
+
+from bzrlib import (
+    bzrdir,
+    lockable_files,
+    lockdir,
+    weave,
+    weavefile,
+    xml5,
+    )
+from bzrlib.decorators import needs_read_lock, needs_write_lock
+from bzrlib.repository import (
+    MetaDirRepository,
+    MetaDirRepositoryFormat,
+    Repository,
+    RepositoryFormat,
+    )
+from bzrlib.store.text import TextStore
+from bzrlib.trace import mutter
+
+
+class AllInOneRepository(Repository):
+    """Legacy support - the repository behaviour for all-in-one branches."""
+
+    _serializer = xml5.serializer_v5
+
+    def __init__(self, _format, a_bzrdir, _revision_store, control_store, text_store):
+        # we reuse one control files instance.
+        dir_mode = a_bzrdir._control_files._dir_mode
+        file_mode = a_bzrdir._control_files._file_mode
+
+        def get_store(name, compressed=True, prefixed=False):
+            # FIXME: This approach of assuming stores are all entirely compressed
+            # or entirely uncompressed is tidy, but breaks upgrade from 
+            # some existing branches where there's a mixture; we probably 
+            # still want the option to look for both.
+            relpath = a_bzrdir._control_files._escape(name)
+            store = TextStore(a_bzrdir._control_files._transport.clone(relpath),
+                              prefixed=prefixed, compressed=compressed,
+                              dir_mode=dir_mode,
+                              file_mode=file_mode)
+            #if self._transport.should_cache():
+            #    cache_path = os.path.join(self.cache_root, name)
+            #    os.mkdir(cache_path)
+            #    store = bzrlib.store.CachedStore(store, cache_path)
+            return store
+
+        # not broken out yet because the controlweaves|inventory_store
+        # and text_store | weave_store bits are still different.
+        if isinstance(_format, RepositoryFormat4):
+            # cannot remove these - there is still no consistent api 
+            # which allows access to this old info.
+            self.inventory_store = get_store('inventory-store')
+            text_store = get_store('text-store')
+        super(AllInOneRepository, self).__init__(_format, a_bzrdir, a_bzrdir._control_files, _revision_store, control_store, text_store)
+
+    def get_commit_builder(self, branch, parents, config, timestamp=None,
+                           timezone=None, committer=None, revprops=None,
+                           revision_id=None):
+        self._check_ascii_revisionid(revision_id, self.get_commit_builder)
+        return Repository.get_commit_builder(self, branch, parents, config,
+            timestamp, timezone, committer, revprops, revision_id)
+
+    @needs_read_lock
+    def is_shared(self):
+        """AllInOne repositories cannot be shared."""
+        return False
+
+    @needs_write_lock
+    def set_make_working_trees(self, new_value):
+        """Set the policy flag for making working trees when creating branches.
+
+        This only applies to branches that use this repository.
+
+        The default is 'True'.
+        :param new_value: True to restore the default, False to disable making
+                          working trees.
+        """
+        raise NotImplementedError(self.set_make_working_trees)
+    
+    def make_working_trees(self):
+        """Returns the policy for making working trees on new branches."""
+        return True
+
+
+class WeaveMetaDirRepository(MetaDirRepository):
+    """A subclass of MetaDirRepository to set weave specific policy."""
+
+    _serializer = xml5.serializer_v5
+
+    def get_commit_builder(self, branch, parents, config, timestamp=None,
+                           timezone=None, committer=None, revprops=None,
+                           revision_id=None):
+        self._check_ascii_revisionid(revision_id, self.get_commit_builder)
+        return MetaDirRepository.get_commit_builder(self, branch, parents,
+            config, timestamp, timezone, committer, revprops, revision_id)
+
+
+class PreSplitOutRepositoryFormat(RepositoryFormat):
+    """Base class for the pre split out repository formats."""
+
+    rich_root_data = False
+
+    def initialize(self, a_bzrdir, shared=False, _internal=False):
+        """Create a weave repository.
+        
+        TODO: when creating split out bzr branch formats, move this to a common
+        base for Format5, Format6. or something like that.
+        """
+        if shared:
+            raise errors.IncompatibleFormat(self, a_bzrdir._format)
+
+        if not _internal:
+            # always initialized when the bzrdir is.
+            return self.open(a_bzrdir, _found=True)
+        
+        # Create an empty weave
+        sio = StringIO()
+        weavefile.write_weave_v5(weave.Weave(), sio)
+        empty_weave = sio.getvalue()
+
+        mutter('creating repository in %s.', a_bzrdir.transport.base)
+        dirs = ['revision-store', 'weaves']
+        files = [('inventory.weave', StringIO(empty_weave)),
+                 ]
+        
+        # FIXME: RBC 20060125 don't peek under the covers
+        # NB: no need to escape relative paths that are url safe.
+        control_files = lockable_files.LockableFiles(a_bzrdir.transport,
+                                'branch-lock', lockable_files.TransportLock)
+        control_files.create_lock()
+        control_files.lock_write()
+        control_files._transport.mkdir_multi(dirs,
+                mode=control_files._dir_mode)
+        try:
+            for file, content in files:
+                control_files.put(file, content)
+        finally:
+            control_files.unlock()
+        return self.open(a_bzrdir, _found=True)
+
+    def _get_control_store(self, repo_transport, control_files):
+        """Return the control store for this repository."""
+        return self._get_versioned_file_store('',
+                                              repo_transport,
+                                              control_files,
+                                              prefixed=False)
+
+    def _get_text_store(self, transport, control_files):
+        """Get a store for file texts for this format."""
+        raise NotImplementedError(self._get_text_store)
+
+    def open(self, a_bzrdir, _found=False):
+        """See RepositoryFormat.open()."""
+        if not _found:
+            # we are being called directly and must probe.
+            raise NotImplementedError
+
+        repo_transport = a_bzrdir.get_repository_transport(None)
+        control_files = a_bzrdir._control_files
+        text_store = self._get_text_store(repo_transport, control_files)
+        control_store = self._get_control_store(repo_transport, control_files)
+        _revision_store = self._get_revision_store(repo_transport, control_files)
+        return AllInOneRepository(_format=self,
+                                  a_bzrdir=a_bzrdir,
+                                  _revision_store=_revision_store,
+                                  control_store=control_store,
+                                  text_store=text_store)
+
+    def check_conversion_target(self, target_format):
+        pass
+
+
+class RepositoryFormat4(PreSplitOutRepositoryFormat):
+    """Bzr repository format 4.
+
+    This repository format has:
+     - flat stores
+     - TextStores for texts, inventories,revisions.
+
+    This format is deprecated: it indexes texts using a text id which is
+    removed in format 5; initialization and write support for this format
+    has been removed.
+    """
+
+    _matchingbzrdir = bzrdir.BzrDirFormat4()
+
+    def __init__(self):
+        super(RepositoryFormat4, self).__init__()
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Repository format 4"
+
+    def initialize(self, url, shared=False, _internal=False):
+        """Format 4 branches cannot be created."""
+        raise errors.UninitializableFormat(self)
+
+    def is_supported(self):
+        """Format 4 is not supported.
+
+        It is not supported because the model changed from 4 to 5 and the
+        conversion logic is expensive - so doing it on the fly was not 
+        feasible.
+        """
+        return False
+
+    def _get_control_store(self, repo_transport, control_files):
+        """Format 4 repositories have no formal control store at this point.
+        
+        This will cause any control-file-needing apis to fail - this is desired.
+        """
+        return None
+    
+    def _get_revision_store(self, repo_transport, control_files):
+        """See RepositoryFormat._get_revision_store()."""
+        from bzrlib.xml4 import serializer_v4
+        return self._get_text_rev_store(repo_transport,
+                                        control_files,
+                                        'revision-store',
+                                        serializer=serializer_v4)
+
+    def _get_text_store(self, transport, control_files):
+        """See RepositoryFormat._get_text_store()."""
+
+
+class RepositoryFormat5(PreSplitOutRepositoryFormat):
+    """Bzr control format 5.
+
+    This repository format has:
+     - weaves for file texts and inventory
+     - flat stores
+     - TextStores for revisions and signatures.
+    """
+
+    _versionedfile_class = weave.WeaveFile
+    _matchingbzrdir = bzrdir.BzrDirFormat5()
+
+    def __init__(self):
+        super(RepositoryFormat5, self).__init__()
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Weave repository format 5"
+
+    def _get_revision_store(self, repo_transport, control_files):
+        """See RepositoryFormat._get_revision_store()."""
+        """Return the revision store object for this a_bzrdir."""
+        return self._get_text_rev_store(repo_transport,
+                                        control_files,
+                                        'revision-store',
+                                        compressed=False)
+
+    def _get_text_store(self, transport, control_files):
+        """See RepositoryFormat._get_text_store()."""
+        return self._get_versioned_file_store('weaves', transport, control_files, prefixed=False)
+
+
+class RepositoryFormat6(PreSplitOutRepositoryFormat):
+    """Bzr control format 6.
+
+    This repository format has:
+     - weaves for file texts and inventory
+     - hash subdirectory based stores.
+     - TextStores for revisions and signatures.
+    """
+
+    _versionedfile_class = weave.WeaveFile
+    _matchingbzrdir = bzrdir.BzrDirFormat6()
+
+    def __init__(self):
+        super(RepositoryFormat6, self).__init__()
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Weave repository format 6"
+
+    def _get_revision_store(self, repo_transport, control_files):
+        """See RepositoryFormat._get_revision_store()."""
+        return self._get_text_rev_store(repo_transport,
+                                        control_files,
+                                        'revision-store',
+                                        compressed=False,
+                                        prefixed=True)
+
+    def _get_text_store(self, transport, control_files):
+        """See RepositoryFormat._get_text_store()."""
+        return self._get_versioned_file_store('weaves', transport, control_files)
+
+
+class RepositoryFormat7(MetaDirRepositoryFormat):
+    """Bzr repository 7.
+
+    This repository format has:
+     - weaves for file texts and inventory
+     - hash subdirectory based stores.
+     - TextStores for revisions and signatures.
+     - a format marker of its own
+     - an optional 'shared-storage' flag
+     - an optional 'no-working-trees' flag
+    """
+
+    _versionedfile_class = weave.WeaveFile
+
+    def _get_control_store(self, repo_transport, control_files):
+        """Return the control store for this repository."""
+        return self._get_versioned_file_store('',
+                                              repo_transport,
+                                              control_files,
+                                              prefixed=False)
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar-NG Repository format 7"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Weave repository format 7"
+
+    def check_conversion_target(self, target_format):
+        pass
+
+    def _get_revision_store(self, repo_transport, control_files):
+        """See RepositoryFormat._get_revision_store()."""
+        return self._get_text_rev_store(repo_transport,
+                                        control_files,
+                                        'revision-store',
+                                        compressed=False,
+                                        prefixed=True,
+                                        )
+
+    def _get_text_store(self, transport, control_files):
+        """See RepositoryFormat._get_text_store()."""
+        return self._get_versioned_file_store('weaves',
+                                              transport,
+                                              control_files)
+
+    def initialize(self, a_bzrdir, shared=False):
+        """Create a weave repository.
+
+        :param shared: If true the repository will be initialized as a shared
+                       repository.
+        """
+        # Create an empty weave
+        sio = StringIO()
+        weavefile.write_weave_v5(weave.Weave(), sio)
+        empty_weave = sio.getvalue()
+
+        mutter('creating repository in %s.', a_bzrdir.transport.base)
+        dirs = ['revision-store', 'weaves']
+        files = [('inventory.weave', StringIO(empty_weave)), 
+                 ]
+        utf8_files = [('format', self.get_format_string())]
+ 
+        self._upload_blank_content(a_bzrdir, dirs, files, utf8_files, shared)
+        return self.open(a_bzrdir=a_bzrdir, _found=True)
+
+    def open(self, a_bzrdir, _found=False, _override_transport=None):
+        """See RepositoryFormat.open().
+        
+        :param _override_transport: INTERNAL USE ONLY. Allows opening the
+                                    repository at a slightly different url
+                                    than normal. I.e. during 'upgrade'.
+        """
+        if not _found:
+            format = RepositoryFormat.find_format(a_bzrdir)
+            assert format.__class__ ==  self.__class__
+        if _override_transport is not None:
+            repo_transport = _override_transport
+        else:
+            repo_transport = a_bzrdir.get_repository_transport(None)
+        control_files = lockable_files.LockableFiles(repo_transport,
+                                'lock', lockdir.LockDir)
+        text_store = self._get_text_store(repo_transport, control_files)
+        control_store = self._get_control_store(repo_transport, control_files)
+        _revision_store = self._get_revision_store(repo_transport, control_files)
+        return WeaveMetaDirRepository(_format=self,
+            a_bzrdir=a_bzrdir,
+            control_files=control_files,
+            _revision_store=_revision_store,
+            control_store=control_store,
+            text_store=text_store)
+
+
+_legacy_formats = [RepositoryFormat4(),
+                   RepositoryFormat5(),
+                   RepositoryFormat6()]

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2007-02-03 10:56:04 +0000
+++ bzrlib/repository.py	2007-02-13 07:28:10 +0000
@@ -18,8 +18,6 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from binascii import hexlify
-from copy import deepcopy
 import re
 import time
 import unittest
@@ -27,12 +25,10 @@
 from bzrlib import (
     bzrdir,
     check,
-    delta,
     errors,
     generate_ids,
     gpg,
     graph,
-    knit,
     lazy_regex,
     lockable_files,
     lockdir,
@@ -42,20 +38,12 @@
     symbol_versioning,
     transactions,
     ui,
-    weave,
-    weavefile,
-    xml5,
-    xml6,
-    )
-from bzrlib.osutils import (
-    rand_bytes,
-    compact_date, 
-    local_time_offset,
     )
 from bzrlib.revisiontree import RevisionTree
 from bzrlib.store.versioned import VersionedFileStore
 from bzrlib.store.text import TextStore
 from bzrlib.testament import Testament
+
 """)
 
 from bzrlib.decorators import needs_read_lock, needs_write_lock
@@ -224,7 +212,6 @@
         # TODO: make sure to construct the right store classes, etc, depending
         # on whether escaping is required.
         self._warn_if_deprecated()
-        self._serializer = xml5.serializer_v5
 
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, 
@@ -354,20 +341,21 @@
 
         Currently no check is made that the format of this repository and
         the bzrdir format are compatible. FIXME RBC 20060201.
+
+        :return: The newly created destination repository.
         """
         if not isinstance(a_bzrdir._format, self.bzrdir._format.__class__):
             # use target default format.
-            result = a_bzrdir.create_repository()
-        # FIXME RBC 20060209 split out the repository type to avoid this check ?
-        elif isinstance(a_bzrdir._format,
-                      (bzrdir.BzrDirFormat4,
-                       bzrdir.BzrDirFormat5,
-                       bzrdir.BzrDirFormat6)):
-            result = a_bzrdir.open_repository()
+            dest_repo = a_bzrdir.create_repository()
         else:
-            result = self._format.initialize(a_bzrdir, shared=self.is_shared())
-        self.copy_content_into(result, revision_id, basis)
-        return result
+            # Most control formats need the repository to be specifically
+            # created, but on some old all-in-one formats it's not needed
+            try:
+                dest_repo = self._format.initialize(a_bzrdir, shared=self.is_shared())
+            except errors.UninitializableFormat:
+                dest_repo = a_bzrdir.open_repository()
+        self.copy_content_into(dest_repo, revision_id, basis)
+        return dest_repo
 
     @needs_read_lock
     def has_revision(self, revision_id):
@@ -829,66 +817,41 @@
                     raise errors.NonAsciiRevisionId(method, self)
 
 
-class AllInOneRepository(Repository):
-    """Legacy support - the repository behaviour for all-in-one branches."""
-
-    def __init__(self, _format, a_bzrdir, _revision_store, control_store, text_store):
-        # we reuse one control files instance.
-        dir_mode = a_bzrdir._control_files._dir_mode
-        file_mode = a_bzrdir._control_files._file_mode
-
-        def get_store(name, compressed=True, prefixed=False):
-            # FIXME: This approach of assuming stores are all entirely compressed
-            # or entirely uncompressed is tidy, but breaks upgrade from 
-            # some existing branches where there's a mixture; we probably 
-            # still want the option to look for both.
-            relpath = a_bzrdir._control_files._escape(name)
-            store = TextStore(a_bzrdir._control_files._transport.clone(relpath),
-                              prefixed=prefixed, compressed=compressed,
-                              dir_mode=dir_mode,
-                              file_mode=file_mode)
-            #if self._transport.should_cache():
-            #    cache_path = os.path.join(self.cache_root, name)
-            #    os.mkdir(cache_path)
-            #    store = bzrlib.store.CachedStore(store, cache_path)
-            return store
-
-        # not broken out yet because the controlweaves|inventory_store
-        # and text_store | weave_store bits are still different.
-        if isinstance(_format, RepositoryFormat4):
-            # cannot remove these - there is still no consistent api 
-            # which allows access to this old info.
-            self.inventory_store = get_store('inventory-store')
-            text_store = get_store('text-store')
-        super(AllInOneRepository, self).__init__(_format, a_bzrdir, a_bzrdir._control_files, _revision_store, control_store, text_store)
-
-    def get_commit_builder(self, branch, parents, config, timestamp=None,
-                           timezone=None, committer=None, revprops=None,
-                           revision_id=None):
-        self._check_ascii_revisionid(revision_id, self.get_commit_builder)
-        return Repository.get_commit_builder(self, branch, parents, config,
-            timestamp, timezone, committer, revprops, revision_id)
-
-    @needs_read_lock
-    def is_shared(self):
-        """AllInOne repositories cannot be shared."""
-        return False
-
-    @needs_write_lock
-    def set_make_working_trees(self, new_value):
-        """Set the policy flag for making working trees when creating branches.
-
-        This only applies to branches that use this repository.
-
-        The default is 'True'.
-        :param new_value: True to restore the default, False to disable making
-                          working trees.
-        """
-        raise NotImplementedError(self.set_make_working_trees)
-    
-    def make_working_trees(self):
-        """Returns the policy for making working trees on new branches."""
-        return True
+
+# remove these delegates a while after bzr 0.15
+def __make_delegated(name, from_module):
+    def _deprecated_repository_forwarder():
+        symbol_versioning.warn('%s moved to %s in bzr 0.15'
+            % (name, from_module),
+            DeprecationWarning,
+            stacklevel=2)
+        m = __import__(from_module, globals(), locals(), [name])
+        try:
+            return getattr(m, name)
+        except AttributeError:
+            raise AttributeError('module %s has no name %s'
+                    % (m, name))
+    globals()[name] = _deprecated_repository_forwarder
+
+for _name in [
+        'AllInOneRepository',
+        'WeaveMetaDirRepository',
+        'PreSplitOutRepositoryFormat',
+        'RepositoryFormat4',
+        'RepositoryFormat5',
+        'RepositoryFormat6',
+        'RepositoryFormat7',
+        ]:
+    __make_delegated(_name, 'bzrlib.repofmt.weaverepo')
+
+for _name in [
+        'KnitRepository',
+        'KnitRepository2',
+        'RepositoryFormatKnit',
+        'RepositoryFormatKnit1',
+        'RepositoryFormatKnit2',
+        ]:
+    __make_delegated(_name, 'bzrlib.repofmt.knitrepo')
 
 
 def install_revision(repository, rev, revision_tree):
@@ -980,219 +943,23 @@
         return not self.control_files._transport.has('no-working-trees')
 
 
-class WeaveMetaDirRepository(MetaDirRepository):
-    """A subclass of MetaDirRepository to set weave specific policy."""
-
-    def get_commit_builder(self, branch, parents, config, timestamp=None,
-                           timezone=None, committer=None, revprops=None,
-                           revision_id=None):
-        self._check_ascii_revisionid(revision_id, self.get_commit_builder)
-        return MetaDirRepository.get_commit_builder(self, branch, parents,
-            config, timestamp, timezone, committer, revprops, revision_id)
-
-
-class KnitRepository(MetaDirRepository):
-    """Knit format repository."""
-
-    def _warn_if_deprecated(self):
-        # This class isn't deprecated
-        pass
-
-    def _inventory_add_lines(self, inv_vf, revid, parents, lines):
-        inv_vf.add_lines_with_ghosts(revid, parents, lines)
-
-    @needs_read_lock
-    def _all_revision_ids(self):
-        """See Repository.all_revision_ids()."""
-        # Knits get the revision graph from the index of the revision knit, so
-        # it's always possible even if they're on an unlistable transport.
-        return self._revision_store.all_revision_ids(self.get_transaction())
-
-    def fileid_involved_between_revs(self, from_revid, to_revid):
-        """Find file_id(s) which are involved in the changes between revisions.
-
-        This determines the set of revisions which are involved, and then
-        finds all file ids affected by those revisions.
-        """
-        vf = self._get_revision_vf()
-        from_set = set(vf.get_ancestry(from_revid))
-        to_set = set(vf.get_ancestry(to_revid))
-        changed = to_set.difference(from_set)
-        return self._fileid_involved_by_set(changed)
-
-    def fileid_involved(self, last_revid=None):
-        """Find all file_ids modified in the ancestry of last_revid.
-
-        :param last_revid: If None, last_revision() will be used.
-        """
-        if not last_revid:
-            changed = set(self.all_revision_ids())
-        else:
-            changed = set(self.get_ancestry(last_revid))
-        if None in changed:
-            changed.remove(None)
-        return self._fileid_involved_by_set(changed)
-
-    @needs_read_lock
-    def get_ancestry(self, revision_id):
-        """Return a list of revision-ids integrated by a revision.
-        
-        This is topologically sorted.
-        """
-        if revision_id is None:
-            return [None]
-        vf = self._get_revision_vf()
-        try:
-            return [None] + vf.get_ancestry(revision_id)
-        except errors.RevisionNotPresent:
-            raise errors.NoSuchRevision(self, revision_id)
-
-    @needs_read_lock
-    def get_revision(self, revision_id):
-        """Return the Revision object for a named revision"""
-        return self.get_revision_reconcile(revision_id)
-
-    @needs_read_lock
-    def get_revision_graph(self, revision_id=None):
-        """Return a dictionary containing the revision graph.
-
-        :param revision_id: The revision_id to get a graph from. If None, then
-        the entire revision graph is returned. This is a deprecated mode of
-        operation and will be removed in the future.
-        :return: a dictionary of revision_id->revision_parents_list.
-        """
-        # special case NULL_REVISION
-        if revision_id == _mod_revision.NULL_REVISION:
-            return {}
-        a_weave = self._get_revision_vf()
-        entire_graph = a_weave.get_graph()
-        if revision_id is None:
-            return a_weave.get_graph()
-        elif revision_id not in a_weave:
-            raise errors.NoSuchRevision(self, revision_id)
-        else:
-            # add what can be reached from revision_id
-            result = {}
-            pending = set([revision_id])
-            while len(pending) > 0:
-                node = pending.pop()
-                result[node] = a_weave.get_parents(node)
-                for revision_id in result[node]:
-                    if revision_id not in result:
-                        pending.add(revision_id)
-            return result
-
-    @needs_read_lock
-    def get_revision_graph_with_ghosts(self, revision_ids=None):
-        """Return a graph of the revisions with ghosts marked as applicable.
-
-        :param revision_ids: an iterable of revisions to graph or None for all.
-        :return: a Graph object with the graph reachable from revision_ids.
-        """
-        result = graph.Graph()
-        vf = self._get_revision_vf()
-        versions = set(vf.versions())
-        if not revision_ids:
-            pending = set(self.all_revision_ids())
-            required = set([])
-        else:
-            pending = set(revision_ids)
-            # special case NULL_REVISION
-            if _mod_revision.NULL_REVISION in pending:
-                pending.remove(_mod_revision.NULL_REVISION)
-            required = set(pending)
-        done = set([])
-        while len(pending):
-            revision_id = pending.pop()
-            if not revision_id in versions:
-                if revision_id in required:
-                    raise errors.NoSuchRevision(self, revision_id)
-                # a ghost
-                result.add_ghost(revision_id)
-                # mark it as done so we don't try for it again.
-                done.add(revision_id)
-                continue
-            parent_ids = vf.get_parents_with_ghosts(revision_id)
-            for parent_id in parent_ids:
-                # is this queued or done ?
-                if (parent_id not in pending and
-                    parent_id not in done):
-                    # no, queue it.
-                    pending.add(parent_id)
-            result.add_node(revision_id, parent_ids)
-            done.add(revision_id)
-        return result
-
-    def _get_revision_vf(self):
-        """:return: a versioned file containing the revisions."""
-        vf = self._revision_store.get_revision_file(self.get_transaction())
-        return vf
-
-    @needs_write_lock
-    def reconcile(self, other=None, thorough=False):
-        """Reconcile this repository."""
-        from bzrlib.reconcile import KnitReconciler
-        reconciler = KnitReconciler(self, thorough=thorough)
-        reconciler.reconcile()
-        return reconciler
-    
-    def revision_parents(self, revision_id):
-        return self._get_revision_vf().get_parents(revision_id)
-
-
-class KnitRepository2(KnitRepository):
-    """"""
-    def __init__(self, _format, a_bzrdir, control_files, _revision_store,
-                 control_store, text_store):
-        KnitRepository.__init__(self, _format, a_bzrdir, control_files,
-                              _revision_store, control_store, text_store)
-        self._serializer = xml6.serializer_v6
-
-    def deserialise_inventory(self, revision_id, xml):
-        """Transform the xml into an inventory object. 
-
-        :param revision_id: The expected revision id of the inventory.
-        :param xml: A serialised inventory.
-        """
-        result = self._serializer.read_inventory_from_string(xml)
-        assert result.root.revision is not None
-        return result
-
-    def serialise_inventory(self, inv):
-        """Transform the inventory object into XML text.
-
-        :param revision_id: The expected revision id of the inventory.
-        :param xml: A serialised inventory.
-        """
-        assert inv.revision_id is not None
-        assert inv.root.revision is not None
-        return KnitRepository.serialise_inventory(self, inv)
-
-    def get_commit_builder(self, branch, parents, config, timestamp=None, 
-                           timezone=None, committer=None, revprops=None, 
-                           revision_id=None):
-        """Obtain a CommitBuilder for this repository.
-        
-        :param branch: Branch to commit to.
-        :param parents: Revision ids of the parents of the new revision.
-        :param config: Configuration to use.
-        :param timestamp: Optional timestamp recorded for commit.
-        :param timezone: Optional timezone for timestamp.
-        :param committer: Optional committer to set for commit.
-        :param revprops: Optional dictionary of revision properties.
-        :param revision_id: Optional revision id.
-        """
-        return RootCommitBuilder(self, parents, config, timestamp, timezone,
-                                 committer, revprops, revision_id)
-
-
 class RepositoryFormatRegistry(registry.Registry):
     """Registry of RepositoryFormats.
     """
+
+    def get(self, format_string):
+        r = registry.Registry.get(self, format_string)
+        if callable(r):
+            r = r()
+        return r
     
 
 format_registry = RepositoryFormatRegistry()
-"""Registry of formats, indexed by their identifying format string."""
+"""Registry of formats, indexed by their identifying format string.
+
+This can contain either format instances themselves, or classes/factories that
+can be called to obtain one.
+"""
 
 
 class RepositoryFormat(object):
@@ -1222,6 +989,10 @@
     def __str__(self):
         return "<%s>" % self.__class__.__name__
 
+    def __eq__(self, other):
+        # format objects are generally stateless
+        return isinstance(other, self.__class__)
+
     @classmethod
     def find_format(klass, a_bzrdir):
         """Return the format for the repository object in a_bzrdir.
@@ -1296,14 +1067,18 @@
         _revision_store = TextRevisionStore(text_store, serializer)
         return _revision_store
 
+    # TODO: this shouldn't be in the base class, it's specific to things that
+    # use weaves or knits -- mbp 20070207
     def _get_versioned_file_store(self,
                                   name,
                                   transport,
                                   control_files,
                                   prefixed=True,
-                                  versionedfile_class=weave.WeaveFile,
+                                  versionedfile_class=None,
                                   versionedfile_kwargs={},
                                   escaped=False):
+        if versionedfile_class is None:
+            versionedfile_class = self._versionedfile_class
         weave_transport = control_files._transport.clone(name)
         dir_mode = control_files._dir_mode
         file_mode = control_files._file_mode
@@ -1344,201 +1119,14 @@
         raise NotImplementedError(self.open)
 
 
-class PreSplitOutRepositoryFormat(RepositoryFormat):
-    """Base class for the pre split out repository formats."""
-
-    rich_root_data = False
-
-    def initialize(self, a_bzrdir, shared=False, _internal=False):
-        """Create a weave repository.
-        
-        TODO: when creating split out bzr branch formats, move this to a common
-        base for Format5, Format6. or something like that.
-        """
-        if shared:
-            raise errors.IncompatibleFormat(self, a_bzrdir._format)
-
-        if not _internal:
-            # always initialized when the bzrdir is.
-            return self.open(a_bzrdir, _found=True)
-        
-        # Create an empty weave
-        sio = StringIO()
-        weavefile.write_weave_v5(weave.Weave(), sio)
-        empty_weave = sio.getvalue()
-
-        mutter('creating repository in %s.', a_bzrdir.transport.base)
-        dirs = ['revision-store', 'weaves']
-        files = [('inventory.weave', StringIO(empty_weave)),
-                 ]
-        
-        # FIXME: RBC 20060125 don't peek under the covers
-        # NB: no need to escape relative paths that are url safe.
-        control_files = lockable_files.LockableFiles(a_bzrdir.transport,
-                                'branch-lock', lockable_files.TransportLock)
-        control_files.create_lock()
-        control_files.lock_write()
-        control_files._transport.mkdir_multi(dirs,
-                mode=control_files._dir_mode)
-        try:
-            for file, content in files:
-                control_files.put(file, content)
-        finally:
-            control_files.unlock()
-        return self.open(a_bzrdir, _found=True)
-
-    def _get_control_store(self, repo_transport, control_files):
-        """Return the control store for this repository."""
-        return self._get_versioned_file_store('',
-                                              repo_transport,
-                                              control_files,
-                                              prefixed=False)
-
-    def _get_text_store(self, transport, control_files):
-        """Get a store for file texts for this format."""
-        raise NotImplementedError(self._get_text_store)
-
-    def open(self, a_bzrdir, _found=False):
-        """See RepositoryFormat.open()."""
-        if not _found:
-            # we are being called directly and must probe.
-            raise NotImplementedError
-
-        repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = a_bzrdir._control_files
-        text_store = self._get_text_store(repo_transport, control_files)
-        control_store = self._get_control_store(repo_transport, control_files)
-        _revision_store = self._get_revision_store(repo_transport, control_files)
-        return AllInOneRepository(_format=self,
-                                  a_bzrdir=a_bzrdir,
-                                  _revision_store=_revision_store,
-                                  control_store=control_store,
-                                  text_store=text_store)
-
-    def check_conversion_target(self, target_format):
-        pass
-
-
-class RepositoryFormat4(PreSplitOutRepositoryFormat):
-    """Bzr repository format 4.
-
-    This repository format has:
-     - flat stores
-     - TextStores for texts, inventories,revisions.
-
-    This format is deprecated: it indexes texts using a text id which is
-    removed in format 5; initialization and write support for this format
-    has been removed.
-    """
-
-    def __init__(self):
-        super(RepositoryFormat4, self).__init__()
-        self._matchingbzrdir = bzrdir.BzrDirFormat4()
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Repository format 4"
-
-    def initialize(self, url, shared=False, _internal=False):
-        """Format 4 branches cannot be created."""
-        raise errors.UninitializableFormat(self)
-
-    def is_supported(self):
-        """Format 4 is not supported.
-
-        It is not supported because the model changed from 4 to 5 and the
-        conversion logic is expensive - so doing it on the fly was not 
-        feasible.
-        """
-        return False
-
-    def _get_control_store(self, repo_transport, control_files):
-        """Format 4 repositories have no formal control store at this point.
-        
-        This will cause any control-file-needing apis to fail - this is desired.
-        """
-        return None
-    
-    def _get_revision_store(self, repo_transport, control_files):
-        """See RepositoryFormat._get_revision_store()."""
-        from bzrlib.xml4 import serializer_v4
-        return self._get_text_rev_store(repo_transport,
-                                        control_files,
-                                        'revision-store',
-                                        serializer=serializer_v4)
-
-    def _get_text_store(self, transport, control_files):
-        """See RepositoryFormat._get_text_store()."""
-
-
-class RepositoryFormat5(PreSplitOutRepositoryFormat):
-    """Bzr control format 5.
-
-    This repository format has:
-     - weaves for file texts and inventory
-     - flat stores
-     - TextStores for revisions and signatures.
-    """
-
-    def __init__(self):
-        super(RepositoryFormat5, self).__init__()
-        self._matchingbzrdir = bzrdir.BzrDirFormat5()
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Weave repository format 5"
-
-    def _get_revision_store(self, repo_transport, control_files):
-        """See RepositoryFormat._get_revision_store()."""
-        """Return the revision store object for this a_bzrdir."""
-        return self._get_text_rev_store(repo_transport,
-                                        control_files,
-                                        'revision-store',
-                                        compressed=False)
-
-    def _get_text_store(self, transport, control_files):
-        """See RepositoryFormat._get_text_store()."""
-        return self._get_versioned_file_store('weaves', transport, control_files, prefixed=False)
-
-
-class RepositoryFormat6(PreSplitOutRepositoryFormat):
-    """Bzr control format 6.
-
-    This repository format has:
-     - weaves for file texts and inventory
-     - hash subdirectory based stores.
-     - TextStores for revisions and signatures.
-    """
-
-    def __init__(self):
-        super(RepositoryFormat6, self).__init__()
-        self._matchingbzrdir = bzrdir.BzrDirFormat6()
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Weave repository format 6"
-
-    def _get_revision_store(self, repo_transport, control_files):
-        """See RepositoryFormat._get_revision_store()."""
-        return self._get_text_rev_store(repo_transport,
-                                        control_files,
-                                        'revision-store',
-                                        compressed=False,
-                                        prefixed=True)
-
-    def _get_text_store(self, transport, control_files):
-        """See RepositoryFormat._get_text_store()."""
-        return self._get_versioned_file_store('weaves', transport, control_files)
-
-
 class MetaDirRepositoryFormat(RepositoryFormat):
     """Common base class for the new repositories using the metadir layout."""
 
     rich_root_data = False
+    _matchingbzrdir = bzrdir.BzrDirMetaFormat1()
 
     def __init__(self):
         super(MetaDirRepositoryFormat, self).__init__()
-        self._matchingbzrdir = bzrdir.BzrDirMetaFormat1()
 
     def _create_control_files(self, a_bzrdir):
         """Create the required files and the initial control_files object."""
@@ -1567,302 +1155,32 @@
             control_files.unlock()
 
 
-class RepositoryFormat7(MetaDirRepositoryFormat):
-    """Bzr repository 7.
-
-    This repository format has:
-     - weaves for file texts and inventory
-     - hash subdirectory based stores.
-     - TextStores for revisions and signatures.
-     - a format marker of its own
-     - an optional 'shared-storage' flag
-     - an optional 'no-working-trees' flag
-    """
-
-    def _get_control_store(self, repo_transport, control_files):
-        """Return the control store for this repository."""
-        return self._get_versioned_file_store('',
-                                              repo_transport,
-                                              control_files,
-                                              prefixed=False)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return "Bazaar-NG Repository format 7"
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Weave repository format 7"
-
-    def check_conversion_target(self, target_format):
-        pass
-
-    def _get_revision_store(self, repo_transport, control_files):
-        """See RepositoryFormat._get_revision_store()."""
-        return self._get_text_rev_store(repo_transport,
-                                        control_files,
-                                        'revision-store',
-                                        compressed=False,
-                                        prefixed=True,
-                                        )
-
-    def _get_text_store(self, transport, control_files):
-        """See RepositoryFormat._get_text_store()."""
-        return self._get_versioned_file_store('weaves',
-                                              transport,
-                                              control_files)
-
-    def initialize(self, a_bzrdir, shared=False):
-        """Create a weave repository.
-
-        :param shared: If true the repository will be initialized as a shared
-                       repository.
-        """
-        # Create an empty weave
-        sio = StringIO()
-        weavefile.write_weave_v5(weave.Weave(), sio)
-        empty_weave = sio.getvalue()
-
-        mutter('creating repository in %s.', a_bzrdir.transport.base)
-        dirs = ['revision-store', 'weaves']
-        files = [('inventory.weave', StringIO(empty_weave)), 
-                 ]
-        utf8_files = [('format', self.get_format_string())]
- 
-        self._upload_blank_content(a_bzrdir, dirs, files, utf8_files, shared)
-        return self.open(a_bzrdir=a_bzrdir, _found=True)
-
-    def open(self, a_bzrdir, _found=False, _override_transport=None):
-        """See RepositoryFormat.open().
-        
-        :param _override_transport: INTERNAL USE ONLY. Allows opening the
-                                    repository at a slightly different url
-                                    than normal. I.e. during 'upgrade'.
-        """
-        if not _found:
-            format = RepositoryFormat.find_format(a_bzrdir)
-            assert format.__class__ ==  self.__class__
-        if _override_transport is not None:
-            repo_transport = _override_transport
-        else:
-            repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = lockable_files.LockableFiles(repo_transport,
-                                'lock', lockdir.LockDir)
-        text_store = self._get_text_store(repo_transport, control_files)
-        control_store = self._get_control_store(repo_transport, control_files)
-        _revision_store = self._get_revision_store(repo_transport, control_files)
-        return WeaveMetaDirRepository(_format=self,
-            a_bzrdir=a_bzrdir,
-            control_files=control_files,
-            _revision_store=_revision_store,
-            control_store=control_store,
-            text_store=text_store)
-
-
-class RepositoryFormatKnit(MetaDirRepositoryFormat):
-    """Bzr repository knit format (generalized). 
-
-    This repository format has:
-     - knits for file texts and inventory
-     - hash subdirectory based stores.
-     - knits for revisions and signatures
-     - TextStores for revisions and signatures.
-     - a format marker of its own
-     - an optional 'shared-storage' flag
-     - an optional 'no-working-trees' flag
-     - a LockDir lock
-    """
-
-    def _get_control_store(self, repo_transport, control_files):
-        """Return the control store for this repository."""
-        return VersionedFileStore(
-            repo_transport,
-            prefixed=False,
-            file_mode=control_files._file_mode,
-            versionedfile_class=knit.KnitVersionedFile,
-            versionedfile_kwargs={'factory':knit.KnitPlainFactory()},
-            )
-
-    def _get_revision_store(self, repo_transport, control_files):
-        """See RepositoryFormat._get_revision_store()."""
-        from bzrlib.store.revision.knit import KnitRevisionStore
-        versioned_file_store = VersionedFileStore(
-            repo_transport,
-            file_mode=control_files._file_mode,
-            prefixed=False,
-            precious=True,
-            versionedfile_class=knit.KnitVersionedFile,
-            versionedfile_kwargs={'delta':False,
-                                  'factory':knit.KnitPlainFactory(),
-                                 },
-            escaped=True,
-            )
-        return KnitRevisionStore(versioned_file_store)
-
-    def _get_text_store(self, transport, control_files):
-        """See RepositoryFormat._get_text_store()."""
-        return self._get_versioned_file_store('knits',
-                                  transport,
-                                  control_files,
-                                  versionedfile_class=knit.KnitVersionedFile,
-                                  versionedfile_kwargs={
-                                      'create_parent_dir':True,
-                                      'delay_create':True,
-                                      'dir_mode':control_files._dir_mode,
-                                  },
-                                  escaped=True)
-
-    def initialize(self, a_bzrdir, shared=False):
-        """Create a knit format 1 repository.
-
-        :param a_bzrdir: bzrdir to contain the new repository; must already
-            be initialized.
-        :param shared: If true the repository will be initialized as a shared
-                       repository.
-        """
-        mutter('creating repository in %s.', a_bzrdir.transport.base)
-        dirs = ['revision-store', 'knits']
-        files = []
-        utf8_files = [('format', self.get_format_string())]
-        
-        self._upload_blank_content(a_bzrdir, dirs, files, utf8_files, shared)
-        repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = lockable_files.LockableFiles(repo_transport,
-                                'lock', lockdir.LockDir)
-        control_store = self._get_control_store(repo_transport, control_files)
-        transaction = transactions.WriteTransaction()
-        # trigger a write of the inventory store.
-        control_store.get_weave_or_empty('inventory', transaction)
-        _revision_store = self._get_revision_store(repo_transport, control_files)
-        # the revision id here is irrelevant: it will not be stored, and cannot
-        # already exist.
-        _revision_store.has_revision_id('A', transaction)
-        _revision_store.get_signature_file(transaction)
-        return self.open(a_bzrdir=a_bzrdir, _found=True)
-
-    def open(self, a_bzrdir, _found=False, _override_transport=None):
-        """See RepositoryFormat.open().
-        
-        :param _override_transport: INTERNAL USE ONLY. Allows opening the
-                                    repository at a slightly different url
-                                    than normal. I.e. during 'upgrade'.
-        """
-        if not _found:
-            format = RepositoryFormat.find_format(a_bzrdir)
-            assert format.__class__ ==  self.__class__
-        if _override_transport is not None:
-            repo_transport = _override_transport
-        else:
-            repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = lockable_files.LockableFiles(repo_transport,
-                                'lock', lockdir.LockDir)
-        text_store = self._get_text_store(repo_transport, control_files)
-        control_store = self._get_control_store(repo_transport, control_files)
-        _revision_store = self._get_revision_store(repo_transport, control_files)
-        return KnitRepository(_format=self,
-                              a_bzrdir=a_bzrdir,
-                              control_files=control_files,
-                              _revision_store=_revision_store,
-                              control_store=control_store,
-                              text_store=text_store)
-
-
-class RepositoryFormatKnit1(RepositoryFormatKnit):
-    """Bzr repository knit format 1.
-
-    This repository format has:
-     - knits for file texts and inventory
-     - hash subdirectory based stores.
-     - knits for revisions and signatures
-     - TextStores for revisions and signatures.
-     - a format marker of its own
-     - an optional 'shared-storage' flag
-     - an optional 'no-working-trees' flag
-     - a LockDir lock
-
-    This format was introduced in bzr 0.8.
-    """
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return "Bazaar-NG Knit Repository Format 1"
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Knit repository format 1"
-
-    def check_conversion_target(self, target_format):
-        pass
-
-
-class RepositoryFormatKnit2(RepositoryFormatKnit):
-    """Bzr repository knit format 2.
-
-    THIS FORMAT IS EXPERIMENTAL
-    This repository format has:
-     - knits for file texts and inventory
-     - hash subdirectory based stores.
-     - knits for revisions and signatures
-     - TextStores for revisions and signatures.
-     - a format marker of its own
-     - an optional 'shared-storage' flag
-     - an optional 'no-working-trees' flag
-     - a LockDir lock
-     - Support for recording full info about the tree root
-
-    """
-    
-    rich_root_data = True
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return "Bazaar Knit Repository Format 2\n"
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Knit repository format 2"
-
-    def check_conversion_target(self, target_format):
-        if not target_format.rich_root_data:
-            raise errors.BadConversionTarget(
-                'Does not support rich root data.', target_format)
-
-    def open(self, a_bzrdir, _found=False, _override_transport=None):
-        """See RepositoryFormat.open().
-        
-        :param _override_transport: INTERNAL USE ONLY. Allows opening the
-                                    repository at a slightly different url
-                                    than normal. I.e. during 'upgrade'.
-        """
-        if not _found:
-            format = RepositoryFormat.find_format(a_bzrdir)
-            assert format.__class__ ==  self.__class__
-        if _override_transport is not None:
-            repo_transport = _override_transport
-        else:
-            repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = lockable_files.LockableFiles(repo_transport, 'lock',
-                                                     lockdir.LockDir)
-        text_store = self._get_text_store(repo_transport, control_files)
-        control_store = self._get_control_store(repo_transport, control_files)
-        _revision_store = self._get_revision_store(repo_transport, control_files)
-        return KnitRepository2(_format=self,
-                               a_bzrdir=a_bzrdir,
-                               control_files=control_files,
-                               _revision_store=_revision_store,
-                               control_store=control_store,
-                               text_store=text_store)
-
-
-
 # formats which have no format string are not discoverable
-# and not independently creatable, so are not registered.
-RepositoryFormat.register_format(RepositoryFormat7())
-# KEEP in sync with bzrdir.format_registry default
-RepositoryFormat.register_format(RepositoryFormatKnit1())
-RepositoryFormat.register_format(RepositoryFormatKnit2())
-_legacy_formats = [RepositoryFormat4(),
-                   RepositoryFormat5(),
-                   RepositoryFormat6()]
+# and not independently creatable, so are not registered.  They're 
+# all in bzrlib.repofmt.weaverepo now.  When an instance of one of these is
+# needed, it's constructed directly by the BzrDir.  Non-native formats where
+# the repository is not separately opened are similar.
+
+format_registry.register_lazy(
+    'Bazaar-NG Repository format 7',
+    'bzrlib.repofmt.weaverepo',
+    'RepositoryFormat7'
+    )
+# KEEP in sync with bzrdir.format_registry default, which controls the overall
+# default control directory format
+
+format_registry.register_lazy(
+    'Bazaar-NG Knit Repository Format 1',
+    'bzrlib.repofmt.knitrepo',
+    'RepositoryFormatKnit1',
+    )
+format_registry.default_key = 'Bazaar-NG Knit Repository Format 1'
+
+format_registry.register_lazy(
+    'Bazaar Knit Repository Format 2\n',
+    'bzrlib.repofmt.knitrepo',
+    'RepositoryFormatKnit2',
+    )
 
 
 class InterRepository(InterObject):
@@ -1928,8 +1246,10 @@
     Data format and model must match for this to work.
     """
 
-    _matching_repo_format = RepositoryFormat4()
-    """Repository format for testing with."""
+    @classmethod
+    def _get_repo_format_to_test(self):
+        """Repository format for testing with."""
+        return RepositoryFormat.get_default_format()
 
     @staticmethod
     def is_compatible(source, target):
@@ -1983,8 +1303,10 @@
 class InterWeaveRepo(InterSameDataRepository):
     """Optimised code paths between Weave based repositories."""
 
-    _matching_repo_format = RepositoryFormat7()
-    """Repository format for testing with."""
+    @classmethod
+    def _get_repo_format_to_test(self):
+        from bzrlib.repofmt import weaverepo
+        return weaverepo.RepositoryFormat7()
 
     @staticmethod
     def is_compatible(source, target):
@@ -1994,6 +1316,11 @@
         could lead to confusing results, and there is no need to be 
         overly general.
         """
+        from bzrlib.repofmt.weaverepo import (
+                RepositoryFormat5,
+                RepositoryFormat6,
+                RepositoryFormat7,
+                )
         try:
             return (isinstance(source._format, (RepositoryFormat5,
                                                 RepositoryFormat6,
@@ -2101,8 +1428,10 @@
 class InterKnitRepo(InterSameDataRepository):
     """Optimised code paths between Knit based repositories."""
 
-    _matching_repo_format = RepositoryFormatKnit1()
-    """Repository format for testing with."""
+    @classmethod
+    def _get_repo_format_to_test(self):
+        from bzrlib.repofmt import knitrepo
+        return knitrepo.RepositoryFormatKnit1()
 
     @staticmethod
     def is_compatible(source, target):
@@ -2112,6 +1441,7 @@
         could lead to confusing results, and there is no need to be 
         overly general.
         """
+        from bzrlib.repofmt.knitrepo import RepositoryFormatKnit1
         try:
             return (isinstance(source._format, (RepositoryFormatKnit1)) and
                     isinstance(target._format, (RepositoryFormatKnit1)))
@@ -2163,7 +1493,9 @@
 
 class InterModel1and2(InterRepository):
 
-    _matching_repo_format = None
+    @classmethod
+    def _get_repo_format_to_test(self):
+        return None
 
     @staticmethod
     def is_compatible(source, target):
@@ -2213,12 +1545,17 @@
 
 class InterKnit1and2(InterKnitRepo):
 
-    _matching_repo_format = None
+    @classmethod
+    def _get_repo_format_to_test(self):
+        return None
 
     @staticmethod
     def is_compatible(source, target):
         """Be compatible with Knit1 source and Knit2 target"""
+        from bzrlib.repofmt.knitrepo import RepositoryFormatKnit2
         try:
+            from bzrlib.repofmt.knitrepo import RepositoryFormatKnit1, \
+                    RepositoryFormatKnit2
             return (isinstance(source._format, (RepositoryFormatKnit1)) and
                     isinstance(target._format, (RepositoryFormatKnit2)))
         except AttributeError:
@@ -2262,6 +1599,7 @@
     def adapt(self, test):
         result = unittest.TestSuite()
         for repository_format, bzrdir_format in self._formats:
+            from copy import deepcopy
             new_test = deepcopy(test)
             new_test.transport_server = self._transport_server
             new_test.transport_readonly_server = self._transport_readonly_server
@@ -2292,6 +1630,7 @@
     def adapt(self, test):
         result = unittest.TestSuite()
         for interrepo_class, repository_format, repository_format_to in self._formats:
+            from copy import deepcopy
             new_test = deepcopy(test)
             new_test.transport_server = self._transport_server
             new_test.transport_readonly_server = self._transport_readonly_server
@@ -2308,6 +1647,7 @@
     @staticmethod
     def default_test_list():
         """Generate the default list of interrepo permutations to test."""
+        from bzrlib.repofmt import knitrepo, weaverepo
         result = []
         # test the default InterRepository between format 6 and the current 
         # default format.
@@ -2316,18 +1656,19 @@
         #result.append((InterRepository,
         #               RepositoryFormat6(),
         #               RepositoryFormatKnit1()))
-        for optimiser in InterRepository._optimisers:
-            if optimiser._matching_repo_format is not None:
-                result.append((optimiser,
-                               optimiser._matching_repo_format,
-                               optimiser._matching_repo_format
-                               ))
+        for optimiser_class in InterRepository._optimisers:
+            format_to_test = optimiser_class._get_repo_format_to_test()
+            if format_to_test is not None:
+                result.append((optimiser_class,
+                               format_to_test, format_to_test))
         # if there are specific combinations we want to use, we can add them 
         # here.
-        result.append((InterModel1and2, RepositoryFormat5(),
-                       RepositoryFormatKnit2()))
-        result.append((InterKnit1and2, RepositoryFormatKnit1(),
-                       RepositoryFormatKnit2()))
+        result.append((InterModel1and2,
+                       weaverepo.RepositoryFormat5(),
+                       knitrepo.RepositoryFormatKnit2()))
+        result.append((InterKnit1and2,
+                       knitrepo.RepositoryFormatKnit1(),
+                       knitrepo.RepositoryFormatKnit2()))
         return result
 
 
@@ -2428,7 +1769,7 @@
         self._timestamp = round(timestamp, 3)
 
         if timezone is None:
-            self._timezone = local_time_offset()
+            self._timezone = osutils.local_time_offset()
         else:
             self._timezone = int(timezone)
 

=== modified file 'bzrlib/shellcomplete.py'
--- bzrlib/shellcomplete.py	2006-12-15 08:36:50 +0000
+++ bzrlib/shellcomplete.py	2007-02-13 01:40:53 +0000
@@ -49,7 +49,7 @@
     for opt in options:
         if opt.short_name:
             outfile.write('"(--%s -%s)"{--%s,-%s}\n'
-                    % (opt.name, opt.short_name, opt.name, opt.short_name))
+                    % (opt.name, opt.short_name(), opt.name, opt.short_name()))
         else:
             outfile.write('--%s\n' % opt.name)
 

=== modified file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	2007-01-05 17:12:03 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2007-02-06 02:33:42 +0000
@@ -349,7 +349,8 @@
 
         bzr('cat-revision', new_rev_id)
 
-    def test_pull_overwrite_fails(self):
+    def test_pull_overwrite(self):
+        # XXX: This test should be moved to branch-implemenations/test_pull
         bzr = self.run_bzr
         self.create_branches()
 
@@ -373,17 +374,8 @@
         self.check_revno(2)
         self.check_revno(2, '../base')
 
-        # It might be possible that we want pull --overwrite to
-        # actually succeed.
-        # If we want it, just change this test to make sure that 
-        # both base and child are updated properly
-        bzr('pull', '--overwrite', '../other', retcode=3)
-
-        # It should fail without changing the local revision
-        self.check_revno(2)
-        self.check_revno(2, '../base')
-
-    # TODO: jam 20051230 Test that commit & pull fail when the branch we 
-    #       are bound to is not available
-
-
+        bzr('pull', '--overwrite', '../other')
+
+        # both the local and master should have been updated.
+        self.check_revno(4)
+        self.check_revno(4, '../base')

=== modified file 'bzrlib/tests/blackbox/test_branch.py'
--- bzrlib/tests/blackbox/test_branch.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/blackbox/test_branch.py	2007-02-07 09:11:31 +0000
@@ -20,7 +20,7 @@
 import os
 
 from bzrlib import branch, bzrdir
-from bzrlib.repository import RepositoryFormatKnit1
+from bzrlib.repofmt.knitrepo import RepositoryFormatKnit1
 from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.workingtree import WorkingTree
 

=== modified file 'bzrlib/tests/blackbox/test_help.py'
--- bzrlib/tests/blackbox/test_help.py	2006-12-14 21:45:21 +0000
+++ bzrlib/tests/blackbox/test_help.py	2007-02-02 00:16:10 +0000
@@ -51,6 +51,12 @@
         self.assertContainsRe(out, 'ancestor:')
         self.assertContainsRe(out, 'branch:')
 
+    def test_help_checkouts(self):
+        """Smoke test for 'bzr help checkouts'"""
+        out, err = self.runbzr('help checkouts')
+        self.assertContainsRe(out, 'checkout')
+        self.assertContainsRe(out, 'lightweight')
+
     def test_help_commands(self):
         dash_help  = self.runbzr('--help commands')[0]
         commands   = self.runbzr('help commands')[0]
@@ -85,3 +91,4 @@
         for line in help.split('\n'):
             if '--long' in line:
                 self.assertTrue('show help on all commands' in line)
+

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2007-01-05 17:12:03 +0000
+++ bzrlib/tests/blackbox/test_info.py	2007-02-06 06:27:24 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
-# -*- coding: utf-8 -*-
+# Copyright (C) 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -485,7 +484,7 @@
         self.assertEqual('', err)
 
     def test_info_shared_repository(self):
-        format = bzrdir.format_registry.make_bzrdir('metaweave')
+        format = bzrdir.format_registry.make_bzrdir('knit')
         transport = self.get_transport()
 
         # Create shared repository
@@ -781,7 +780,7 @@
         self.assertEqual('', err)
 
     def test_info_shared_repository_with_trees(self):
-        format = bzrdir.format_registry.make_bzrdir('metaweave')
+        format = bzrdir.format_registry.make_bzrdir('knit')
         transport = self.get_transport()
 
         # Create shared repository with working trees
@@ -1008,7 +1007,7 @@
         self.assertEqual('', err)
     
     def test_info_shared_repository_with_tree_in_root(self):
-        format = bzrdir.format_registry.make_bzrdir('metaweave')
+        format = bzrdir.format_registry.make_bzrdir('knit')
         transport = self.get_transport()
 
         # Create shared repository with working trees

=== modified file 'bzrlib/tests/blackbox/test_log.py'
--- bzrlib/tests/blackbox/test_log.py	2006-10-30 14:56:33 +0000
+++ bzrlib/tests/blackbox/test_log.py	2007-02-09 18:23:21 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,7 @@
 
 import bzrlib
 from bzrlib.tests.blackbox import ExternalBase
-from bzrlib.tests import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir, TestCaseWithTransport
 
 
 class TestLog(ExternalBase):
@@ -290,3 +290,14 @@
         # Make sure the cp1251 string is not found anywhere
         self.assertEquals(-1, stdout.find(test_in_cp1251))
 
+
+class TestLogFile(TestCaseWithTransport):
+
+    def test_log_local_branch_file(self):
+        """We should be able to log files in local treeless branches"""
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file'])
+        tree.add('file')
+        tree.commit('revision 1')
+        tree.bzrdir.destroy_workingtree()
+        self.run_bzr('log', 'tree/file')

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2007-02-09 17:09:47 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2007-02-14 21:57:40 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -232,13 +232,14 @@
         tree_a.rename_one('file_1', 'file_i')
         tree_a.commit('commit 2')
         tree_a.rename_one('file_2', 'file_ii')
-        os.chdir('b')
-        self.run_bzr('merge', '../a', '--uncommitted')
-        self.failUnlessExists('file_1')
-        self.failUnlessExists('file_ii')
+        ## os.chdir('b')
+        self.run_bzr('merge', 'a', '--uncommitted', '-d', 'b')
+        self.failUnlessExists('b/file_1')
+        self.failUnlessExists('b/file_ii')
         tree_b.revert([])
-        self.run_bzr_error(('Cannot use --uncommitted and --revision',), 
-                           'merge', '../a', '--uncommitted', '-r1')
+        self.run_bzr_error(('Cannot use --uncommitted and --revision',),
+                           'merge', '/a', '--uncommitted', '-r1',
+                           '-d', 'b')
 
     def pullable_branch(self):
         os.mkdir('a')

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2007-02-09 17:09:47 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2007-02-14 21:57:40 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
-# -*- coding: utf-8 -*-
+# Copyright (C) 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +23,7 @@
 from bzrlib.branch import Branch
 from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.uncommit import uncommit
+from bzrlib import urlutils
 
 
 class TestPull(ExternalBase):
@@ -96,6 +96,19 @@
         self.runbzr('pull ../b')
         self.runbzr('pull ../b')
 
+    def test_pull_dash_d(self):
+        os.mkdir('a')
+        os.chdir('a')
+        self.example_branch()
+        self.runbzr('init ../b')
+        self.runbzr('init ../c')
+        # pull into that branch
+        self.runbzr('pull -d ../b .')
+        # pull into a branch specified by a url
+        c_url = urlutils.local_path_to_url('../c')
+        self.assertStartsWith(c_url, 'file://')
+        self.runbzr('pull -d %s .' % c_url)
+
     def test_pull_revision(self):
         """Pull some changes from one branch to another."""
         os.mkdir('a')

=== modified file 'bzrlib/tests/blackbox/test_push.py'
--- bzrlib/tests/blackbox/test_push.py	2006-10-16 01:25:46 +0000
+++ bzrlib/tests/blackbox/test_push.py	2007-02-14 09:51:20 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
-# -*- coding: utf-8 -*-
+# Copyright (C) 2005, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,11 +19,14 @@
 
 import os
 
-import bzrlib
+from bzrlib import (
+    errors,
+    urlutils,
+    )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDirMetaFormat1
 from bzrlib.osutils import abspath
-from bzrlib.repository import RepositoryFormatKnit1
+from bzrlib.repofmt.knitrepo import RepositoryFormatKnit1
 from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.uncommit import uncommit
 from bzrlib.urlutils import local_path_from_url
@@ -97,7 +99,7 @@
         out, err = self.run_bzr('push', 'pushed-location')
         self.assertEqual('', out)
         self.assertEqual('0 revision(s) pushed.\n', err)
-        b2 = bzrlib.branch.Branch.open('pushed-location')
+        b2 = Branch.open('pushed-location')
         self.assertEndsWith(b2.base, 'pushed-location/')
 
     def test_push_new_branch_revision_count(self):
@@ -160,3 +162,83 @@
         t.add('filename', 'funky-chars<>%&;"\'')
         t.commit('commit filename')
         self.run_bzr('push', '../new-tree')
+
+    def test_push_dash_d(self):
+        t = self.make_branch_and_tree('from')
+        t.commit(allow_pointless=True,
+                message='first commit')
+        self.runbzr('push -d from to-one')
+        self.failUnlessExists('to-one')
+        self.runbzr('push -d %s %s' 
+            % tuple(map(urlutils.local_path_to_url, ['from', 'to-two'])))
+        self.failUnlessExists('to-two')
+
+    def create_simple_tree(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add(['a'], ['a-id'])
+        tree.commit('one', rev_id='r1')
+        return tree
+
+    def test_push_create_prefix(self):
+        """'bzr push --create-prefix' will create leading directories."""
+        tree = self.create_simple_tree()
+
+        self.run_bzr_error(['Parent directory of ../new/tree does not exist'],
+                           'push', '../new/tree',
+                           working_dir='tree')
+        self.run_bzr('push', '../new/tree', '--create-prefix',
+                     working_dir='tree')
+        new_tree = WorkingTree.open('new/tree')
+        self.assertEqual(tree.last_revision(), new_tree.last_revision())
+        self.failUnlessExists('new/tree/a')
+
+    def test_push_use_existing(self):
+        """'bzr push --use-existing-dir' can push into an existing dir.
+
+        By default, 'bzr push' will not use an existing, non-versioned dir.
+        """
+        tree = self.create_simple_tree()
+        self.build_tree(['target/'])
+
+        self.run_bzr_error(['Target directory ../target already exists',
+                            'Supply --use-existing-dir',
+                           ], 'push', '../target',
+                           working_dir='tree')
+
+        self.run_bzr('push', '--use-existing-dir', '../target',
+                     working_dir='tree')
+
+        new_tree = WorkingTree.open('target')
+        self.assertEqual(tree.last_revision(), new_tree.last_revision())
+        # The push should have created target/a
+        self.failUnlessExists('target/a')
+
+    def test_push_onto_repo(self):
+        """We should be able to 'bzr push' into an existing bzrdir."""
+        tree = self.create_simple_tree()
+        repo = self.make_repository('repo', shared=True)
+
+        self.run_bzr('push', '../repo',
+                     working_dir='tree')
+
+        # Pushing onto an existing bzrdir will create a repository and
+        # branch as needed, but will only create a working tree if there was
+        # no BzrDir before.
+        self.assertRaises(errors.NoWorkingTree, WorkingTree.open, 'repo')
+        new_branch = Branch.open('repo')
+        self.assertEqual(tree.last_revision(), new_branch.last_revision())
+
+    def test_push_onto_just_bzrdir(self):
+        """We don't handle when the target is just a bzrdir.
+
+        Because you shouldn't be able to create *just* a bzrdir in the wild.
+        """
+        # TODO: jam 20070109 Maybe it would be better to create the repository
+        #       if at this point
+        tree = self.create_simple_tree()
+        a_bzrdir = self.make_bzrdir('dir')
+
+        self.run_bzr_error(['At ../dir you have a valid .bzr control'],
+                'push', '../dir',
+                working_dir='tree')

=== modified file 'bzrlib/tests/blackbox/test_selftest.py'
--- bzrlib/tests/blackbox/test_selftest.py	2006-12-18 09:26:03 +0000
+++ bzrlib/tests/blackbox/test_selftest.py	2007-02-07 15:38:35 +0000
@@ -262,7 +262,8 @@
         self.assertContainsRe(result[1], 'unknown command')
         err = self.run_bzr_subprocess('merge', '--merge-type', 'magic merge', 
                                       retcode=3)[1]
-        self.assertContainsRe(err, 'No known merge type magic merge')
+        self.assertContainsRe(err, 'Bad value "magic merge" for option'
+                              ' "merge-type"')
 
     def test_run_bzr_subprocess_env(self):
         """run_bzr_subprocess can set environment variables in the child only.

=== modified file 'bzrlib/tests/blackbox/test_upgrade.py'
--- bzrlib/tests/blackbox/test_upgrade.py	2007-01-05 17:12:03 +0000
+++ bzrlib/tests/blackbox/test_upgrade.py	2007-02-07 09:11:31 +0000
@@ -28,6 +28,10 @@
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.transport import get_transport
 import bzrlib.ui as ui
+from bzrlib.repofmt.knitrepo import (
+    RepositoryFormatKnit1,
+    RepositoryFormatKnit2,
+    )
 
 
 class TestWithUpgradableBranches(TestCaseWithTransport):
@@ -140,7 +144,7 @@
         self.assertTrue(isinstance(converted_dir._format,
                                    bzrdir.BzrDirMetaFormat1))
         self.assertTrue(isinstance(converted_dir.open_repository()._format,
-                                   repository.RepositoryFormatKnit1))
+                                   RepositoryFormatKnit1))
 
     def test_upgrade_repo(self):
         self.run_bzr('init-repository', '--format=metaweave', 'repo')

=== modified file 'bzrlib/tests/branch_implementations/__init__.py'
--- bzrlib/tests/branch_implementations/__init__.py	2007-02-02 09:17:23 +0000
+++ bzrlib/tests/branch_implementations/__init__.py	2007-02-06 02:33:42 +0000
@@ -42,6 +42,7 @@
         'bzrlib.tests.branch_implementations.test_bound_sftp',
         'bzrlib.tests.branch_implementations.test_branch',
         'bzrlib.tests.branch_implementations.test_break_lock',
+        'bzrlib.tests.branch_implementations.test_commit',
         'bzrlib.tests.branch_implementations.test_hooks',
         'bzrlib.tests.branch_implementations.test_http',
         'bzrlib.tests.branch_implementations.test_last_revision_info',
@@ -50,6 +51,7 @@
         'bzrlib.tests.branch_implementations.test_permissions',
         'bzrlib.tests.branch_implementations.test_pull',
         'bzrlib.tests.branch_implementations.test_push',
+        'bzrlib.tests.branch_implementations.test_uncommit',
         'bzrlib.tests.branch_implementations.test_update',
         ]
     adapter = BranchTestProviderAdapter(

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2007-01-11 17:20:25 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2007-02-06 02:33:42 +0000
@@ -327,17 +327,6 @@
         branch.nick = u"\u1234"
         self.assertEqual(branch.nick, u"\u1234")
 
-    def test_commit_nicks(self):
-        """Nicknames are committed to the revision"""
-        get_transport(self.get_url()).mkdir('bzr.dev')
-        wt = self.make_branch_and_tree('bzr.dev')
-        branch = wt.branch
-        branch.nick = "My happy branch"
-        wt.commit('My commit respect da nick.')
-        committed = branch.repository.get_revision(branch.last_revision())
-        self.assertEqual(committed.properties["branch-nick"], 
-                         "My happy branch")
-
     def test_create_open_branch_uses_repository(self):
         try:
             repo = self.make_repository('.', shared=True)

=== added file 'bzrlib/tests/branch_implementations/test_commit.py'
--- bzrlib/tests/branch_implementations/test_commit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_commit.py	2007-02-06 02:33:42 +0000
@@ -0,0 +1,114 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the contract of commit on branches."""
+
+from bzrlib.branch import Branch
+from bzrlib import errors
+from bzrlib.tests.branch_implementations.test_branch import TestCaseWithBranch
+from bzrlib.revision import NULL_REVISION
+from bzrlib.transport import get_transport
+
+
+class TestCommit(TestCaseWithBranch):
+
+    def test_commit_nicks(self):
+        """Nicknames are committed to the revision"""
+        get_transport(self.get_url()).mkdir('bzr.dev')
+        wt = self.make_branch_and_tree('bzr.dev')
+        branch = wt.branch
+        branch.nick = "My happy branch"
+        wt.commit('My commit respect da nick.')
+        committed = branch.repository.get_revision(branch.last_revision())
+        self.assertEqual(committed.properties["branch-nick"], 
+                         "My happy branch")
+
+
+class TestCommitHook(TestCaseWithBranch):
+
+    def setUp(self):
+        self.hook_calls = []
+        TestCaseWithBranch.setUp(self)
+
+    def capture_post_commit_hook(self, local, master, old_revno,
+        old_revid, new_revno, new_revid):
+        """Capture post commit hook calls to self.hook_calls.
+        
+        The call is logged, as is some state of the two branches.
+        """
+        if local:
+            local_locked = local.is_locked()
+            local_base = local.base
+        else:
+            local_locked = None
+            local_base = None
+        self.hook_calls.append(
+            ('post_commit', local_base, master.base, old_revno, old_revid,
+             new_revno, new_revid, local_locked, master.is_locked()))
+
+    def test_post_commit_to_origin(self):
+        tree = self.make_branch_and_memory_tree('branch')
+        Branch.hooks.install_hook('post_commit',
+            self.capture_post_commit_hook)
+        tree.lock_write()
+        tree.add('')
+        revid = tree.commit('a revision')
+        # should have had one notification, from origin, and
+        # have the branch locked at notification time.
+        self.assertEqual([
+            ('post_commit', None, tree.branch.base, 0, NULL_REVISION, 1, revid,
+             None, True)
+            ],
+            self.hook_calls)
+        tree.unlock()
+
+    def test_post_commit_bound(self):
+        master = self.make_branch('master')
+        tree = self.make_branch_and_memory_tree('local')
+        try:
+            tree.branch.bind(master)
+        except errors.UpgradeRequired:
+            # cant bind this format, the test is irrelevant.
+            return
+        Branch.hooks.install_hook('post_commit',
+            self.capture_post_commit_hook)
+        tree.lock_write()
+        tree.add('')
+        revid = tree.commit('a revision')
+        # with a bound branch, local is set.
+        self.assertEqual([
+            ('post_commit', tree.branch.base, master.base, 0, NULL_REVISION,
+             1, revid, True, True)
+            ],
+            self.hook_calls)
+        tree.unlock()
+
+    def test_post_commit_not_to_origin(self):
+        tree = self.make_branch_and_memory_tree('branch')
+        tree.lock_write()
+        tree.add('')
+        revid = tree.commit('first revision')
+        Branch.hooks.install_hook('post_commit',
+            self.capture_post_commit_hook)
+        revid2 = tree.commit('second revision')
+        # having committed from up the branch, we should get the
+        # before and after revnos and revids correctly.
+        self.assertEqual([
+            ('post_commit', None, tree.branch.base, 1, revid, 2, revid2,
+             None, True)
+            ],
+            self.hook_calls)
+        tree.unlock()

=== modified file 'bzrlib/tests/branch_implementations/test_hooks.py'
--- bzrlib/tests/branch_implementations/test_hooks.py	2007-01-30 10:41:04 +0000
+++ bzrlib/tests/branch_implementations/test_hooks.py	2007-02-06 02:33:42 +0000
@@ -20,48 +20,47 @@
 from bzrlib.tests import TestCaseWithMemoryTransport
 
 
-class TestPushHook(TestCaseWithMemoryTransport):
+class TestSetRevisionHistoryHook(TestCaseWithMemoryTransport):
 
     def setUp(self):
         self.hook_calls = []
         TestCaseWithMemoryTransport.setUp(self)
 
     def capture_set_rh_hook(self, branch, rev_history):
-        """Capture post push hook calls to self.hook_calls.
+        """Capture post set-rh hook calls to self.hook_calls.
         
-        The call is logged, as is some state of the two branches.
+        The call is logged, as is some state of the branch.
         """
         self.hook_calls.append(
             ('set_rh', branch, rev_history, branch.is_locked()))
 
     def test_set_rh_empty_history(self):
         branch = self.make_branch('source')
-        Branch.hooks['set_rh'].append(self.capture_set_rh_hook)
+        Branch.hooks.install_hook('set_rh', self.capture_set_rh_hook)
         branch.set_revision_history([])
         self.assertEqual(self.hook_calls,
             [('set_rh', branch, [], True)])
 
     def test_set_rh_nonempty_history(self):
         branch = self.make_branch('source')
-        Branch.hooks['set_rh'].append(self.capture_set_rh_hook)
+        Branch.hooks.install_hook('set_rh', self.capture_set_rh_hook)
         branch.set_revision_history([u'foo'])
         self.assertEqual(self.hook_calls,
             [('set_rh', branch, [u'foo'], True)])
 
     def test_set_rh_branch_is_locked(self):
         branch = self.make_branch('source')
-        Branch.hooks['set_rh'].append(self.capture_set_rh_hook)
+        Branch.hooks.install_hook('set_rh', self.capture_set_rh_hook)
         branch.set_revision_history([])
         self.assertEqual(self.hook_calls,
             [('set_rh', branch, [], True)])
 
     def test_set_rh_calls_all_hooks_no_errors(self):
         branch = self.make_branch('source')
-        Branch.hooks['set_rh'].append(self.capture_set_rh_hook)
-        Branch.hooks['set_rh'].append(self.capture_set_rh_hook)
+        Branch.hooks.install_hook('set_rh', self.capture_set_rh_hook)
+        Branch.hooks.install_hook('set_rh', self.capture_set_rh_hook)
         branch.set_revision_history([])
         self.assertEqual(self.hook_calls,
             [('set_rh', branch, [], True),
              ('set_rh', branch, [], True),
             ])
-

=== modified file 'bzrlib/tests/branch_implementations/test_pull.py'
--- bzrlib/tests/branch_implementations/test_pull.py	2007-01-30 20:58:25 +0000
+++ bzrlib/tests/branch_implementations/test_pull.py	2007-02-06 02:33:42 +0000
@@ -20,10 +20,12 @@
 
 from bzrlib.branch import Branch
 from bzrlib import errors
-from bzrlib.tests import TestCaseWithTransport
-
-
-class TestPull(TestCaseWithTransport):
+from bzrlib.memorytree import MemoryTree
+from bzrlib.revision import NULL_REVISION
+from bzrlib.tests.branch_implementations.test_branch import TestCaseWithBranch
+
+
+class TestPull(TestCaseWithBranch):
 
     def test_pull_convergence_simple(self):
         # when revisions are pulled, the left-most accessible parents must 
@@ -77,3 +79,82 @@
         # try to pull, which should raise a BoundBranchConnectionFailure.
         self.assertRaises(errors.BoundBranchConnectionFailure,
                 checkout.branch.pull, other.branch)
+
+
+class TestPullHook(TestCaseWithBranch):
+
+    def setUp(self):
+        self.hook_calls = []
+        TestCaseWithBranch.setUp(self)
+
+    def capture_post_pull_hook(self, source, local, master, old_revno,
+        old_revid, new_revno, new_revid):
+        """Capture post pull hook calls to self.hook_calls.
+        
+        The call is logged, as is some state of the two branches.
+        """
+        if local:
+            local_locked = local.is_locked()
+            local_base = local.base
+        else:
+            local_locked = None
+            local_base = None
+        self.hook_calls.append(
+            ('post_pull', source, local_base, master.base, old_revno, old_revid,
+             new_revno, new_revid, source.is_locked(), local_locked,
+             master.is_locked()))
+
+    def test_post_pull_empty_history(self):
+        target = self.make_branch('target')
+        source = self.make_branch('source')
+        Branch.hooks.install_hook('post_pull', self.capture_post_pull_hook)
+        target.pull(source)
+        # with nothing there we should still get a notification, and
+        # have both branches locked at the notification time.
+        self.assertEqual([
+            ('post_pull', source, None, target.base, 0, NULL_REVISION,
+             0, NULL_REVISION, True, None, True)
+            ],
+            self.hook_calls)
+
+    def test_post_pull_bound_branch(self):
+        # pulling to a bound branch should pass in the master branch to the
+        # hook, allowing the correct number of emails to be sent, while still
+        # allowing hooks that want to modify the target to do so to both 
+        # instances.
+        target = self.make_branch('target')
+        local = self.make_branch('local')
+        try:
+            local.bind(target)
+        except errors.UpgradeRequired:
+            # cant bind this format, the test is irrelevant.
+            return
+        source = self.make_branch('source')
+        Branch.hooks.install_hook('post_pull', self.capture_post_pull_hook)
+        local.pull(source)
+        # with nothing there we should still get a notification, and
+        # have both branches locked at the notification time.
+        self.assertEqual([
+            ('post_pull', source, local.base, target.base, 0, NULL_REVISION,
+             0, NULL_REVISION, True, True, True)
+            ],
+            self.hook_calls)
+
+    def test_post_pull_nonempty_history(self):
+        target = self.make_branch_and_memory_tree('target')
+        target.lock_write()
+        target.add('')
+        rev1 = target.commit('rev 1')
+        target.unlock()
+        sourcedir = target.bzrdir.clone(self.get_url('source'))
+        source = MemoryTree.create_on_branch(sourcedir.open_branch())
+        rev2 = source.commit('rev 2')
+        Branch.hooks.install_hook('post_pull', self.capture_post_pull_hook)
+        target.branch.pull(source.branch)
+        # with nothing there we should still get a notification, and
+        # have both branches locked at the notification time.
+        self.assertEqual([
+            ('post_pull', source.branch, None, target.branch.base, 1, rev1,
+             2, rev2, True, None, True)
+            ],
+            self.hook_calls)

=== modified file 'bzrlib/tests/branch_implementations/test_push.py'
--- bzrlib/tests/branch_implementations/test_push.py	2007-01-30 20:58:25 +0000
+++ bzrlib/tests/branch_implementations/test_push.py	2007-02-12 22:58:48 +0000
@@ -20,10 +20,12 @@
 
 from bzrlib.branch import Branch
 from bzrlib import errors
-from bzrlib.tests import TestCaseWithTransport
-
-
-class TestPush(TestCaseWithTransport):
+from bzrlib.memorytree import MemoryTree
+from bzrlib.revision import NULL_REVISION
+from bzrlib.tests.branch_implementations.test_branch import TestCaseWithBranch
+
+
+class TestPush(TestCaseWithBranch):
 
     def test_push_convergence_simple(self):
         # when revisions are pushed, the left-most accessible parents must 
@@ -57,8 +59,13 @@
     def test_push_to_checkout_updates_master(self):
         """Pushing into a checkout updates the checkout and the master branch"""
         master_tree = self.make_branch_and_tree('master')
-        rev1 = master_tree.commit('master')
-        checkout = master_tree.branch.create_checkout('checkout')
+        checkout = self.make_branch_and_tree('checkout')
+        try:
+            checkout.branch.bind(master_tree.branch)
+        except errors.UpgradeRequired:
+            # cant bind this format, the test is irrelevant.
+            return
+        rev1 = checkout.commit('master')
 
         other = master_tree.branch.bzrdir.sprout('other').open_workingtree()
         rev2 = other.commit('other commit')
@@ -69,7 +76,12 @@
 
     def test_push_raises_specific_error_on_master_connection_error(self):
         master_tree = self.make_branch_and_tree('master')
-        checkout = master_tree.branch.create_checkout('checkout')
+        checkout = self.make_branch_and_tree('checkout')
+        try:
+            checkout.branch.bind(master_tree.branch)
+        except errors.UpgradeRequired:
+            # cant bind this format, the test is irrelevant.
+            return
         other = master_tree.branch.bzrdir.sprout('other').open_workingtree()
         # move the branch out of the way on disk to cause a connection
         # error.
@@ -77,3 +89,128 @@
         # try to push, which should raise a BoundBranchConnectionFailure.
         self.assertRaises(errors.BoundBranchConnectionFailure,
                 other.branch.push, checkout.branch)
+
+    def test_push_uses_read_lock(self):
+        """Push should only need a read lock on the source side."""
+        source = self.make_branch_and_tree('source')
+        target = self.make_branch('target')
+
+        self.build_tree(['source/a'])
+        source.add(['a'])
+        source.commit('a')
+
+        source.branch.lock_read()
+        try:
+            target.lock_write()
+            try:
+                source.branch.push(target, stop_revision=source.last_revision())
+            finally:
+                target.unlock()
+        finally:
+            source.branch.unlock()
+
+    def test_push_within_repository(self):
+        """Push from one branch to another inside the same repository."""
+        try:
+            repo = self.make_repository('repo', shared=True)
+        except (errors.IncompatibleFormat, errors.UninitializableFormat):
+            # This Branch format cannot create shared repositories
+            return
+        # This is a little bit trickier because make_branch_and_tree will not
+        # re-use a shared repository.
+        a_bzrdir = self.make_bzrdir('repo/tree')
+        try:
+            a_branch = self.branch_format.initialize(a_bzrdir)
+        except (errors.UninitializableFormat):
+            # Cannot create these branches
+            return
+        tree = a_branch.bzrdir.create_workingtree()
+        self.build_tree(['repo/tree/a'])
+        tree.add(['a'])
+        tree.commit('a')
+
+        to_bzrdir = self.make_bzrdir('repo/branch')
+        to_branch = self.branch_format.initialize(to_bzrdir)
+        tree.branch.push(to_branch)
+
+        self.assertEqual(tree.branch.last_revision(),
+                         to_branch.last_revision())
+
+
+class TestPushHook(TestCaseWithBranch):
+
+    def setUp(self):
+        self.hook_calls = []
+        TestCaseWithBranch.setUp(self)
+
+    def capture_post_push_hook(self, source, local, master, old_revno,
+        old_revid, new_revno, new_revid):
+        """Capture post push hook calls to self.hook_calls.
+        
+        The call is logged, as is some state of the two branches.
+        """
+        if local:
+            local_locked = local.is_locked()
+            local_base = local.base
+        else:
+            local_locked = None
+            local_base = None
+        self.hook_calls.append(
+            ('post_push', source, local_base, master.base, old_revno, old_revid,
+             new_revno, new_revid, source.is_locked(), local_locked,
+             master.is_locked()))
+
+    def test_post_push_empty_history(self):
+        target = self.make_branch('target')
+        source = self.make_branch('source')
+        Branch.hooks.install_hook('post_push', self.capture_post_push_hook)
+        source.push(target)
+        # with nothing there we should still get a notification, and
+        # have both branches locked at the notification time.
+        self.assertEqual([
+            ('post_push', source, None, target.base, 0, NULL_REVISION,
+             0, NULL_REVISION, True, None, True)
+            ],
+            self.hook_calls)
+
+    def test_post_push_bound_branch(self):
+        # pushing to a bound branch should pass in the master branch to the
+        # hook, allowing the correct number of emails to be sent, while still
+        # allowing hooks that want to modify the target to do so to both 
+        # instances.
+        target = self.make_branch('target')
+        local = self.make_branch('local')
+        try:
+            local.bind(target)
+        except errors.UpgradeRequired:
+            # cant bind this format, the test is irrelevant.
+            return
+        source = self.make_branch('source')
+        Branch.hooks.install_hook('post_push', self.capture_post_push_hook)
+        source.push(local)
+        # with nothing there we should still get a notification, and
+        # have both branches locked at the notification time.
+        self.assertEqual([
+            ('post_push', source, local.base, target.base, 0, NULL_REVISION,
+             0, NULL_REVISION, True, True, True)
+            ],
+            self.hook_calls)
+
+    def test_post_push_nonempty_history(self):
+        target = self.make_branch_and_memory_tree('target')
+        target.lock_write()
+        target.add('')
+        rev1 = target.commit('rev 1')
+        target.unlock()
+        sourcedir = target.bzrdir.clone(self.get_url('source'))
+        source = MemoryTree.create_on_branch(sourcedir.open_branch())
+        rev2 = source.commit('rev 2')
+        Branch.hooks.install_hook('post_push', self.capture_post_push_hook)
+        source.branch.push(target.branch)
+        # with nothing there we should still get a notification, and
+        # have both branches locked at the notification time.
+        self.assertEqual([
+            ('post_push', source.branch, None, target.branch.base, 1, rev1,
+             2, rev2, True, None, True)
+            ],
+            self.hook_calls)

=== added file 'bzrlib/tests/branch_implementations/test_uncommit.py'
--- bzrlib/tests/branch_implementations/test_uncommit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_uncommit.py	2007-02-06 02:33:42 +0000
@@ -0,0 +1,108 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the contract of uncommit on branches.
+
+Note that uncommit currently is not a branch method; it should be.
+"""
+
+from bzrlib.branch import Branch
+from bzrlib import errors
+from bzrlib.tests.branch_implementations.test_branch import TestCaseWithBranch
+from bzrlib.uncommit import uncommit
+
+
+class TestUncommitHook(TestCaseWithBranch):
+
+    def setUp(self):
+        self.hook_calls = []
+        TestCaseWithBranch.setUp(self)
+
+    def capture_post_uncommit_hook(self, local, master, old_revno,
+        old_revid, new_revno, new_revid):
+        """Capture post uncommit hook calls to self.hook_calls.
+        
+        The call is logged, as is some state of the two branches.
+        """
+        if local:
+            local_locked = local.is_locked()
+            local_base = local.base
+        else:
+            local_locked = None
+            local_base = None
+        self.hook_calls.append(
+            ('post_uncommit', local_base, master.base, old_revno, old_revid,
+             new_revno, new_revid, local_locked, master.is_locked()))
+
+    def test_post_uncommit_to_origin(self):
+        tree = self.make_branch_and_memory_tree('branch')
+        tree.lock_write()
+        tree.add('')
+        revid = tree.commit('a revision')
+        tree.unlock()
+        Branch.hooks.install_hook('post_uncommit',
+            self.capture_post_uncommit_hook)
+        uncommit(tree.branch)
+        # with nothing left we should still get a notification, and
+        # have the branch locked at notification time.
+        self.assertEqual([
+            ('post_uncommit', None, tree.branch.base, 1, revid,
+             0, None, None, True)
+            ],
+            self.hook_calls)
+
+    def test_post_uncommit_bound(self):
+        master = self.make_branch('master')
+        tree = self.make_branch_and_memory_tree('local')
+        try:
+            tree.branch.bind(master)
+        except errors.UpgradeRequired:
+            # cant bind this format, the test is irrelevant.
+            return
+        tree.lock_write()
+        tree.add('')
+        revid = tree.commit('a revision')
+        tree.unlock()
+        Branch.hooks.install_hook('post_uncommit',
+            self.capture_post_uncommit_hook)
+        uncommit(tree.branch)
+        # with nothing left we should still get a notification, and
+        # have the branch locked at notification time.
+        self.assertEqual([
+            ('post_uncommit', tree.branch.base, master.base, 1, revid,
+             0, None, True, True)
+            ],
+            self.hook_calls)
+
+    def test_post_uncommit_not_to_origin(self):
+        tree = self.make_branch_and_memory_tree('branch')
+        tree.lock_write()
+        tree.add('')
+        revid = tree.commit('first revision')
+        revid2 = tree.commit('second revision')
+        revid3 = tree.commit('third revision')
+        tree.unlock()
+        Branch.hooks.install_hook('post_uncommit',
+            self.capture_post_uncommit_hook)
+        uncommit(tree.branch, revno=2)
+        # having uncommitted from up the branch, we should get the
+        # before and after revnos and revids correctly.
+        self.assertEqual([
+            ('post_uncommit', None, tree.branch.base, 3, revid3,
+             1, revid, None, True)
+            ],
+            self.hook_calls)
+

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2007-01-09 06:57:43 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2007-02-06 06:27:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -51,6 +51,7 @@
 from bzrlib.trace import mutter
 from bzrlib.transport import get_transport
 from bzrlib.upgrade import upgrade
+from bzrlib.repofmt import weaverepo
 
 
 class TestCaseWithBzrDir(TestCaseWithTransport):
@@ -1106,9 +1107,9 @@
         self.assertTrue(isinstance(dir.get_repository_transport(None),
                                    transport.Transport))
         # with a given format, either the bzr dir supports identifiable
-        # repositoryes, or it supports anonymous  repository formats, but not both.
-        anonymous_format = repository.RepositoryFormat6()
-        identifiable_format = repository.RepositoryFormat7()
+        # repositories, or it supports anonymous  repository formats, but not both.
+        anonymous_format = weaverepo.RepositoryFormat6()
+        identifiable_format = weaverepo.RepositoryFormat7()
         try:
             found_transport = dir.get_repository_transport(anonymous_format)
             self.assertRaises(errors.IncompatibleFormat,

=== modified file 'bzrlib/tests/interrepository_implementations/__init__.py'
--- bzrlib/tests/interrepository_implementations/__init__.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/interrepository_implementations/__init__.py	2007-02-06 06:27:24 +0000
@@ -25,9 +25,9 @@
 rather than in tests/interrepository_implementations/*.py.
 """
 
-from bzrlib.repository import (_legacy_formats,
-                               InterRepositoryTestProviderAdapter,
-                               )
+from bzrlib.repository import (
+    InterRepositoryTestProviderAdapter,
+    )
                             
 from bzrlib.tests import (
                           adapt_modules,

=== modified file 'bzrlib/tests/interrepository_implementations/test_interrepository.py'
--- bzrlib/tests/interrepository_implementations/test_interrepository.py	2007-02-03 07:36:05 +0000
+++ bzrlib/tests/interrepository_implementations/test_interrepository.py	2007-02-13 05:22:39 +0000
@@ -112,10 +112,16 @@
         # we assume the optimising code paths are triggered
         # by the type of the repo not the transport - at this point.
         # we may need to update this test if this changes.
-        source_repo = self.make_repository("source")
-        target_repo = self.make_to_repository("target")
-        interrepo = repository.InterRepository.get(source_repo, target_repo)
-        self.assertEqual(self.interrepo_class, interrepo.__class__)
+        #
+        # XXX: This code tests that we get an InterRepository when we try to
+        # convert between the two repositories that it wants to be tested with
+        # -- but that's not necessarily correct.  So for now this is disabled.
+        # mbp 20070206
+        ## source_repo = self.make_repository("source")
+        ## target_repo = self.make_to_repository("target")
+        ## interrepo = repository.InterRepository.get(source_repo, target_repo)
+        ## self.assertEqual(self.interrepo_class, interrepo.__class__)
+        pass
 
     def test_fetch(self):
         tree_a = self.make_branch_and_tree('a')

=== modified file 'bzrlib/tests/repository_implementations/__init__.py'
--- bzrlib/tests/repository_implementations/__init__.py	2007-02-03 10:02:11 +0000
+++ bzrlib/tests/repository_implementations/__init__.py	2007-02-13 05:22:39 +0000
@@ -27,10 +27,12 @@
 from bzrlib import (
     repository,
     )
-from bzrlib.repository import (_legacy_formats,
-                               RepositoryTestProviderAdapter,
-                               )
-                            
+from bzrlib.repository import (
+    RepositoryTestProviderAdapter,
+    )
+from bzrlib.repofmt import (
+    weaverepo,
+    )
 from bzrlib.tests import (
                           adapt_modules,
                           default_transport,
@@ -50,8 +52,9 @@
         'bzrlib.tests.repository_implementations.test_revision',
         'bzrlib.tests.repository_implementations.test_statistics',
         ]
-    all_formats = [v for (k, v) in repository.format_registry.iteritems()]
-    all_formats.extend(_legacy_formats)
+    registry = repository.format_registry
+    all_formats = [registry.get(k) for k in registry.keys()]
+    all_formats.extend(weaverepo._legacy_formats)
     adapter = RepositoryTestProviderAdapter(
         default_transport,
         # None here will cause a readonly decorator to be created

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2007-01-17 15:37:08 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2007-02-06 08:16:13 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,12 @@
 import sys
 
 import bzrlib
-from bzrlib import bzrdir, errors, repository
+from bzrlib import (
+    bzrdir,
+    errors,
+    repository,
+    transactions,
+    )
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
 from bzrlib.delta import TreeDelta
 from bzrlib.errors import (FileExists,
@@ -32,10 +37,10 @@
                            )
 from bzrlib.inventory import Inventory, InventoryDirectory
 from bzrlib.revision import NULL_REVISION
+from bzrlib.repofmt import knitrepo
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
 from bzrlib.trace import mutter
-import bzrlib.transactions as transactions
 from bzrlib.transport import get_transport
 from bzrlib.upgrade import upgrade
 from bzrlib.workingtree import WorkingTree
@@ -190,7 +195,7 @@
         tree_a.commit('rev1', rev_id='rev1')
         # fetch with a default limit (grab everything)
         f = bzrdir.BzrDirMetaFormat1()
-        f._repository_format = repository.RepositoryFormatKnit2()
+        f._repository_format = knitrepo.RepositoryFormatKnit2()
         os.mkdir('b')
         b_bzrdir = f.initialize(self.get_url('b'))
         repo = b_bzrdir.create_repository()
@@ -392,7 +397,8 @@
 
     def test_upgrade_from_format4(self):
         from bzrlib.tests.test_upgrade import _upgrade_dir_template
-        if self.repository_format.__class__ == repository.RepositoryFormat4:
+        if self.repository_format.get_format_description() \
+            == "Repository format 4":
             raise TestSkipped('Cannot convert format-4 to itself')
         self.build_tree_contents(_upgrade_dir_template)
         old_repodir = bzrlib.bzrdir.BzrDir.open_unsupported('.')

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2007-01-30 11:52:30 +0000
+++ bzrlib/tests/test_branch.py	2007-02-06 02:33:42 +0000
@@ -171,6 +171,10 @@
         """Check that creating a BranchHooks instance has the right defaults."""
         hooks = bzrlib.branch.BranchHooks()
         self.assertTrue("set_rh" in hooks, "set_rh not in %s" % hooks)
+        self.assertTrue("post_push" in hooks, "post_push not in %s" % hooks)
+        self.assertTrue("post_commit" in hooks, "post_commit not in %s" % hooks)
+        self.assertTrue("post_pull" in hooks, "post_pull not in %s" % hooks)
+        self.assertTrue("post_uncommit" in hooks, "post_uncommit not in %s" % hooks)
 
     def test_installed_hooks_are_BranchHooks(self):
         """The installed hooks object should be a BranchHooks."""

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2006-11-02 10:20:19 +0000
+++ bzrlib/tests/test_bundle.py	2007-02-07 09:11:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2006 Canonical Ltd
+# Copyright (C) 2004, 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 from bzrlib.errors import (BzrError, TestamentMismatch, NotABundle, BadBundle, 
                            NoSuchFile,)
 from bzrlib.merge import Merge3Merger
+from bzrlib.repofmt import knitrepo
 from bzrlib.osutils import has_symlinks, sha_file
 from bzrlib.tests import (TestCaseInTempDir, TestCaseWithTransport,
                           TestCase, TestSkipped)
@@ -313,7 +314,7 @@
 
     def test_mismatched_bundle(self):
         format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormatKnit2()
+        format.repository_format = knitrepo.RepositoryFormatKnit2()
         serializer = BundleSerializerV08('0.8')
         b = self.make_branch('.', format=format)
         self.assertRaises(errors.IncompatibleBundleFormat, serializer.write, 
@@ -322,7 +323,7 @@
     def test_matched_bundle(self):
         """Don't raise IncompatibleBundleFormat for knit2 and bundle0.9"""
         format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormatKnit2()
+        format.repository_format = knitrepo.RepositoryFormatKnit2()
         serializer = BundleSerializerV09('0.9')
         b = self.make_branch('.', format=format)
         serializer.write(b.repository, [], {}, StringIO())
@@ -330,7 +331,7 @@
     def test_mismatched_model(self):
         """Try copying a bundle from knit2 to knit1"""
         format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormatKnit2()
+        format.repository_format = knitrepo.RepositoryFormatKnit2()
         source = self.make_branch_and_tree('source', format=format)
         source.commit('one', rev_id='one-id')
         source.commit('two', rev_id='two-id')
@@ -340,7 +341,7 @@
         text.seek(0)
 
         format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormatKnit1()
+        format.repository_format = knitrepo.RepositoryFormatKnit1()
         target = self.make_branch('target', format=format)
         self.assertRaises(errors.IncompatibleRevision, install_bundle, 
                           target.repository, read_bundle(text))
@@ -352,7 +353,7 @@
 
     def bzrdir_format(self):
         format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormatKnit1()
+        format.repository_format = knitrepo.RepositoryFormatKnit1()
         return format
 
     def make_branch_and_tree(self, path, format=None):
@@ -890,7 +891,7 @@
 
     def bzrdir_format(self):
         format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormatKnit2()
+        format.repository_format = knitrepo.RepositoryFormatKnit2()
         return format
 
 
@@ -900,7 +901,7 @@
 
     def bzrdir_format(self):
         format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormatKnit1()
+        format.repository_format = knitrepo.RepositoryFormatKnit1()
         return format
 
 

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2007-01-17 14:36:49 +0000
+++ bzrlib/tests/test_bzrdir.py	2007-02-14 07:10:27 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
     help_topics,
     symbol_versioning,
     urlutils,
+    workingtree,
     )
 import bzrlib.branch
 import bzrlib.bzrdir as bzrdir
@@ -39,7 +40,7 @@
 from bzrlib.tests.HttpServer import HttpServer
 from bzrlib.transport import get_transport
 from bzrlib.transport.memory import MemoryServer
-import bzrlib.workingtree as workingtree
+from bzrlib.repofmt import knitrepo, weaverepo
 
 
 class TestDefaultFormat(TestCase):
@@ -70,14 +71,16 @@
             ' repositories', deprecated=True)
         my_format_registry.register_lazy('lazy', 'bzrlib.bzrdir', 
             'BzrDirFormat6', 'Format registered lazily', deprecated=True)
-        my_format_registry.register_metadir('knit', 'RepositoryFormatKnit1',
-            'Format using knits')
+        my_format_registry.register_metadir('knit',
+            'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
+            'Format using knits',
+            )
         my_format_registry.set_default('knit')
-        my_format_registry.register_metadir('metaweave', 'RepositoryFormat7',
-            'Transitional format in 0.8.  Slower than knit.', deprecated=True)
-        my_format_registry.register_metadir('experimental-knit2', 
-                                            'RepositoryFormatKnit2',
-            'Experimental successor to knit.  Use at your own risk.')
+        my_format_registry.register_metadir(
+            'experimental-knit2',
+            'bzrlib.repofmt.knitrepo.RepositoryFormatKnit2',
+            'Experimental successor to knit.  Use at your own risk.',
+            )
         return my_format_registry
 
     def test_format_registry(self):
@@ -88,13 +91,10 @@
         self.assertIsInstance(my_bzrdir, bzrdir.BzrDirFormat6)
         my_bzrdir = my_format_registry.make_bzrdir('default')
         self.assertIsInstance(my_bzrdir.repository_format, 
-            repository.RepositoryFormatKnit1)
+            knitrepo.RepositoryFormatKnit1)
         my_bzrdir = my_format_registry.make_bzrdir('knit')
         self.assertIsInstance(my_bzrdir.repository_format, 
-            repository.RepositoryFormatKnit1)
-        my_bzrdir = my_format_registry.make_bzrdir('metaweave')
-        self.assertIsInstance(my_bzrdir.repository_format, 
-            repository.RepositoryFormat7)
+            knitrepo.RepositoryFormatKnit1)
 
     def test_get_help(self):
         my_format_registry = self.make_format_registry()
@@ -124,13 +124,13 @@
         default_factory = bzrdir.format_registry.get('default')
         old_default = [k for k, v in bzrdir.format_registry.iteritems()
                        if v == default_factory and k != 'default'][0]
-        bzrdir.format_registry.set_default_repository('metaweave')
+        bzrdir.format_registry.set_default_repository('experimental-knit2')
         try:
-            self.assertIs(bzrdir.format_registry.get('metaweave'),
+            self.assertIs(bzrdir.format_registry.get('experimental-knit2'),
                           bzrdir.format_registry.get('default'))
             self.assertIs(
                 repository.RepositoryFormat.get_default_format().__class__,
-                repository.RepositoryFormat7)
+                knitrepo.RepositoryFormatKnit2)
         finally:
             bzrdir.format_registry.set_default_repository(old_default)
 
@@ -482,7 +482,7 @@
         repository_base = t.clone('repository').base
         self.assertEqual(repository_base, dir.get_repository_transport(None).base)
         self.assertEqual(repository_base,
-                         dir.get_repository_transport(repository.RepositoryFormat7()).base)
+                         dir.get_repository_transport(weaverepo.RepositoryFormat7()).base)
         checkout_base = t.clone('checkout').base
         self.assertEqual(checkout_base, dir.get_workingtree_transport(None).base)
         self.assertEqual(checkout_base,

=== modified file 'bzrlib/tests/test_fetch.py'
--- bzrlib/tests/test_fetch.py	2006-11-08 07:44:30 +0000
+++ bzrlib/tests/test_fetch.py	2007-02-07 09:11:31 +0000
@@ -23,6 +23,7 @@
 from bzrlib.bzrdir import BzrDir
 from bzrlib.builtins import merge
 import bzrlib.errors
+from bzrlib.repofmt import knitrepo
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
 from bzrlib.tests.test_revision import make_branches
@@ -123,9 +124,9 @@
         corresponding filename, parent, contents or other changes.
         """
         knit1_format = bzrdir.BzrDirMetaFormat1()
-        knit1_format.repository_format = repository.RepositoryFormatKnit1()
+        knit1_format.repository_format = knitrepo.RepositoryFormatKnit1()
         knit2_format = bzrdir.BzrDirMetaFormat1()
-        knit2_format.repository_format = repository.RepositoryFormatKnit2()
+        knit2_format.repository_format = knitrepo.RepositoryFormatKnit2()
         # we start with a knit1 repository because that causes the
         # root revision to change for each commit, even though the content,
         # parent, name, and other attributes are unchanged.

=== modified file 'bzrlib/tests/test_gpg.py'
--- bzrlib/tests/test_gpg.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/test_gpg.py	2007-02-08 23:40:24 +0000
@@ -80,6 +80,11 @@
             ui.ui_factory.clear_term = old_clear_term
         self.assertEqual([True], clear_term_called)
 
+    def test_aborts_on_unicode(self):
+        """You can't sign Unicode text; it must be encoded first."""
+        self.assertRaises(errors.BzrBadParameterUnicode,
+                          self.assertProduces, u'foo')
+
 
 class TestDisabled(TestCase):
 

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2006-12-15 02:47:54 +0000
+++ bzrlib/tests/test_http.py	2007-02-08 17:01:20 +0000
@@ -636,6 +636,57 @@
     """Tests range requests refusing server for pycurl implementation"""
 
 
+class TestHttpProxyWhiteBox(TestCase):
+    """Whitebox test proxy http authorization."""
+
+    def setUp(self):
+        TestCase.setUp(self)
+        self._old_env = {}
+
+    def tearDown(self):
+        self._restore_env()
+
+    def _set_and_capture_env_var(self, name, new_value):
+        """Set an environment variable, and reset it when finished."""
+        self._old_env[name] = osutils.set_or_unset_env(name, new_value)
+
+    def _install_env(self, env):
+        for name, value in env.iteritems():
+            self._set_and_capture_env_var(name, value)
+
+    def _restore_env(self):
+        for name, value in self._old_env.iteritems():
+            osutils.set_or_unset_env(name, value)
+
+    def _proxied_request(self):
+        from bzrlib.transport.http._urllib2_wrappers import (
+            ProxyHandler,
+            Request,
+            )
+
+        handler = ProxyHandler()
+        request = Request('GET','http://baz/buzzle')
+        handler.set_proxy(request, 'http')
+        return request
+
+    def test_empty_user(self):
+        self._install_env({'http_proxy': 'http://bar.com'})
+        request = self._proxied_request()
+        self.assertFalse(request.headers.has_key('Proxy-authorization'))
+
+    def test_empty_pass(self):
+        self._install_env({'http_proxy': 'http://joe@bar.com'})
+        request = self._proxied_request()
+        self.assertEqual('Basic ' + 'joe:'.encode('base64').strip(),
+                         request.headers['Proxy-authorization'])
+    def test_user_pass(self):
+        self._install_env({'http_proxy': 'http://joe:foo@bar.com'})
+        request = self._proxied_request()
+        self.assertEqual('Basic ' + 'joe:foo'.encode('base64').strip(),
+                         request.headers['Proxy-authorization'])
+
+
+
 class TestProxyHttpServer(object):
     """Tests proxy server.
 
@@ -655,6 +706,10 @@
     # FIXME: We don't have an https server available, so we don't
     # test https connections.
 
+    # FIXME: Once the test suite is better fitted to test
+    # authorization schemes, test proxy authorizations too (see
+    # bug #83954).
+
     def setUp(self):
         TestCaseWithTwoWebservers.setUp(self)
         self.build_tree_contents([('foo', 'contents of foo\n'),

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2007-02-06 15:32:30 +0000
+++ bzrlib/tests/test_merge.py	2007-02-07 16:05:09 +0000
@@ -17,7 +17,11 @@
 import os
 from StringIO import StringIO
 
-from bzrlib import conflicts
+from bzrlib import (
+    conflicts,
+    merge as _mod_merge,
+    option,
+    )
 from bzrlib.branch import Branch
 from bzrlib.builtins import merge
 from bzrlib.conflicts import ConflictList, TextConflict
@@ -206,3 +210,16 @@
         self.assertEqual(tree_b.conflicts(),
                          [conflicts.ContentsConflict('file',
                           file_id='file-id')])
+    
+    def test_merge_type_registry(self):
+        merge_type_option = option.Option.OPTIONS['merge-type']
+        self.assertFalse('merge4' in [x[0] for x in 
+                        merge_type_option.iter_switches()])
+        registry = _mod_merge.get_merge_type_registry()
+        registry.register_lazy('merge4', 'bzrlib.merge', 'Merge4Merger',
+                               'time-travelling merge')
+        self.assertTrue('merge4' in [x[0] for x in 
+                        merge_type_option.iter_switches()])
+        registry.remove('merge4')
+        self.assertFalse('merge4' in [x[0] for x in 
+                        merge_type_option.iter_switches()])

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2007-02-07 03:40:37 +0000
+++ bzrlib/tests/test_options.py	2007-02-14 07:10:27 +0000
@@ -25,6 +25,7 @@
 from bzrlib.builtins import cmd_commit, cmd_log, cmd_status
 from bzrlib.commands import Command, parse_args
 from bzrlib.tests import TestCase
+from bzrlib.repofmt import knitrepo
 
 def parse(options, args):
     parser = option.get_optparser(dict((o.name, o) for o in options))
@@ -162,12 +163,15 @@
                    bzrdir.format_registry, bzrdir.format_registry.make_bzrdir)]
         opts, args = self.parse(options, ['--format', 'knit'])
         self.assertIsInstance(opts.format.repository_format,
-                              repository.RepositoryFormatKnit1)
+                              knitrepo.RepositoryFormatKnit1)
 
     def test_help(self):
         registry = bzrdir.BzrDirFormatRegistry()
         registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help')
+        registry.register_metadir('two',
+            'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
+            'two help',
+            )
         registry.set_default('one')
         options = [option.RegistryOption('format', 'format help', registry,
                    str, value_switches=True, title='Formats')]
@@ -189,7 +193,10 @@
                          [('hello', None, 'GAR', 'fg')])
         registry = bzrdir.BzrDirFormatRegistry()
         registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help')
+        registry.register_metadir('two',
+                'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
+                'two help',
+                )
         registry.set_default('one')
         opt = option.RegistryOption('format', 'format help', registry,
                                     value_switches=False)

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2007-01-26 21:56:16 +0000
+++ bzrlib/tests/test_repository.py	2007-02-13 07:28:10 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@
     upgrade,
     workingtree,
     )
+from bzrlib.repofmt import knitrepo, weaverepo
 
 
 class TestDefaultFormat(TestCase):
@@ -111,7 +112,7 @@
             t = get_transport(url)
             found_format = repository.RepositoryFormat.find_format(dir)
             self.failUnless(isinstance(found_format, format.__class__))
-        check_format(repository.RepositoryFormat7(), "bar")
+        check_format(weaverepo.RepositoryFormat7(), "bar")
         
     def test_find_format_no_repository(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
@@ -146,7 +147,7 @@
 
     def test_no_ancestry_weave(self):
         control = bzrdir.BzrDirFormat6().initialize(self.get_url())
-        repo = repository.RepositoryFormat6().initialize(control)
+        repo = weaverepo.RepositoryFormat6().initialize(control)
         # We no longer need to create the ancestry.weave file
         # since it is *never* used.
         self.assertRaises(NoSuchFile,
@@ -158,7 +159,7 @@
     
     def test_disk_layout(self):
         control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = repository.RepositoryFormat7().initialize(control)
+        repo = weaverepo.RepositoryFormat7().initialize(control)
         # in case of side effects of locking.
         repo.lock_write()
         repo.unlock()
@@ -180,7 +181,7 @@
 
     def test_shared_disk_layout(self):
         control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = repository.RepositoryFormat7().initialize(control, shared=True)
+        repo = weaverepo.RepositoryFormat7().initialize(control, shared=True)
         # we want:
         # format 'Bazaar-NG Repository format 7'
         # inventory.weave == empty_weave
@@ -203,7 +204,7 @@
     def test_creates_lockdir(self):
         """Make sure it appears to be controlled by a LockDir existence"""
         control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = repository.RepositoryFormat7().initialize(control, shared=True)
+        repo = weaverepo.RepositoryFormat7().initialize(control, shared=True)
         t = control.get_repository_transport(None)
         # TODO: Should check there is a 'lock' toplevel directory, 
         # regardless of contents
@@ -219,7 +220,7 @@
         """repo format 7 actually locks on lockdir"""
         base_url = self.get_url()
         control = bzrdir.BzrDirMetaFormat1().initialize(base_url)
-        repo = repository.RepositoryFormat7().initialize(control, shared=True)
+        repo = weaverepo.RepositoryFormat7().initialize(control, shared=True)
         t = control.get_repository_transport(None)
         repo.lock_write()
         repo.unlock()
@@ -233,7 +234,7 @@
 
     def test_shared_no_tree_disk_layout(self):
         control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = repository.RepositoryFormat7().initialize(control, shared=True)
+        repo = weaverepo.RepositoryFormat7().initialize(control, shared=True)
         repo.set_make_working_trees(False)
         # we want:
         # format 'Bazaar-NG Repository format 7'
@@ -262,7 +263,7 @@
     
     def test_disk_layout(self):
         control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = repository.RepositoryFormatKnit1().initialize(control)
+        repo = knitrepo.RepositoryFormatKnit1().initialize(control)
         # in case of side effects of locking.
         repo.lock_write()
         repo.unlock()
@@ -295,7 +296,7 @@
 
     def test_shared_disk_layout(self):
         control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = repository.RepositoryFormatKnit1().initialize(control, shared=True)
+        repo = knitrepo.RepositoryFormatKnit1().initialize(control, shared=True)
         # we want:
         # format 'Bazaar-NG Knit Repository Format 1'
         # lock: is a directory
@@ -314,7 +315,7 @@
 
     def test_shared_no_tree_disk_layout(self):
         control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = repository.RepositoryFormatKnit1().initialize(control, shared=True)
+        repo = knitrepo.RepositoryFormatKnit1().initialize(control, shared=True)
         repo.set_make_working_trees(False)
         # we want:
         # format 'Bazaar-NG Knit Repository Format 1'
@@ -402,11 +403,12 @@
     def test_is_compatible_and_registered(self):
         # InterWeaveRepo is compatible when either side
         # is a format 5/6/7 branch
-        formats = [repository.RepositoryFormat5(),
-                   repository.RepositoryFormat6(),
-                   repository.RepositoryFormat7()]
-        incompatible_formats = [repository.RepositoryFormat4(),
-                                repository.RepositoryFormatKnit1(),
+        from bzrlib.repofmt import knitrepo, weaverepo
+        formats = [weaverepo.RepositoryFormat5(),
+                   weaverepo.RepositoryFormat6(),
+                   weaverepo.RepositoryFormat7()]
+        incompatible_formats = [weaverepo.RepositoryFormat4(),
+                                knitrepo.RepositoryFormatKnit1(),
                                 ]
         repo_a = self.make_repository('a')
         repo_b = self.make_repository('b')
@@ -433,8 +435,8 @@
         t = get_transport(self.get_url('.'))
         t.mkdir('repository')
         repo_dir = bzrdir.BzrDirMetaFormat1().initialize('repository')
-        repo = repository.RepositoryFormat7().initialize(repo_dir)
-        target_format = repository.RepositoryFormatKnit1()
+        repo = weaverepo.RepositoryFormat7().initialize(repo_dir)
+        target_format = knitrepo.RepositoryFormatKnit1()
         converter = repository.CopyConverter(target_format)
         pb = bzrlib.ui.ui_factory.nested_progress_bar()
         try:
@@ -457,14 +459,14 @@
     def test_convert(self):
         """Ensure the upgrade adds weaves for roots"""
         format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormatKnit1()
+        format.repository_format = knitrepo.RepositoryFormatKnit1()
         tree = self.make_branch_and_tree('.', format)
         tree.commit("Dull commit", rev_id="dull")
         revision_tree = tree.branch.repository.revision_tree('dull')
         self.assertRaises(errors.NoSuchFile, revision_tree.get_file_lines,
             revision_tree.inventory.root.file_id)
         format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormatKnit2()
+        format.repository_format = knitrepo.RepositoryFormatKnit2()
         upgrade.Convert('.', format)
         tree = workingtree.WorkingTree.open('.')
         revision_tree = tree.branch.repository.revision_tree('dull')

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2007-01-30 10:41:04 +0000
+++ bzrlib/tests/test_selftest.py	2007-02-13 05:22:39 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,11 +27,15 @@
 import bzrlib
 from bzrlib import (
     bzrdir,
+    errors,
     memorytree,
     osutils,
     repository,
+    symbol_versioning,
     )
 from bzrlib.progress import _BaseProgressBar
+from bzrlib.repofmt import weaverepo
+from bzrlib.symbol_versioning import zero_ten, zero_eleven
 from bzrlib.tests import (
                           ChrootedTestCase,
                           TestCase,
@@ -44,9 +48,6 @@
                           )
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.tests.TestUtil import _load_module_by_name
-import bzrlib.errors as errors
-from bzrlib import symbol_versioning
-from bzrlib.symbol_versioning import zero_ten, zero_eleven
 from bzrlib.trace import note
 from bzrlib.transport.memory import MemoryServer, MemoryTransport
 from bzrlib.version import _get_bzr_source_tree
@@ -475,7 +476,7 @@
     def test_make_branch_and_memory_tree_with_format(self):
         """make_branch_and_memory_tree should accept a format option."""
         format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormat7()
+        format.repository_format = weaverepo.RepositoryFormat7()
         tree = self.make_branch_and_memory_tree('dir', format=format)
         # Guard against regression into MemoryTransport leaking
         # files to disk instead of keeping them in memory.

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-12-15 23:34:42 +0000
+++ bzrlib/transport/__init__.py	2007-01-26 01:23:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -93,6 +93,8 @@
     intended to be used when the implementation depends on an external
     implementation that may not be present.  If any other error is raised, it
     propagates up and the attempt to open the url fails.
+
+    :param scheme: The url scheme part, eg "ftp://"
     """
     # TODO: If no implementation of a protocol is available because of missing
     # dependencies, we should perhaps show the message about what dependency
@@ -1004,12 +1006,9 @@
 
     base is either a URL or a directory name.  
     """
-    # TODO: give a better error if base looks like a url but there's no
-    # handler for the scheme?
     global _protocol_handlers
     if base is None:
         base = '.'
-
     last_err = None
 
     def convert_path_to_url(base, error_str):

=== modified file 'bzrlib/transport/http/_urllib2_wrappers.py'
--- bzrlib/transport/http/_urllib2_wrappers.py	2006-12-13 15:08:39 +0000
+++ bzrlib/transport/http/_urllib2_wrappers.py	2007-02-08 17:01:20 +0000
@@ -708,14 +708,17 @@
             user_pass, host = host.split('@', 1)
             if ':' in user_pass:
                 user, password = user_pass.split(':', 1)
-                user_pass = '%s:%s' % (urllib.unquote(user),
-                               urllib.unquote(password))
-                user_pass.encode('base64').strip()
-                req.add_header('Proxy-authorization', 'Basic ' + user_pass)
+            else:
+                user = user_pass
+                password = ''
+            user_pass = '%s:%s' % (urllib.unquote(user),
+                                   urllib.unquote(password))
+            user_pass = user_pass.encode('base64').strip()
+            request.add_header('Proxy-authorization', 'Basic ' + user_pass)
         host = urllib.unquote(host)
         request.set_proxy(host, type)
         if self._debuglevel > 0:
-            print 'set_proxy: proxy set to %r://%r' % (type, host)
+            print 'set_proxy: proxy set to %s://%s' % (type, host)
         return request
 
 

=== modified file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	2006-09-05 09:49:40 +0000
+++ bzrlib/uncommit.py	2007-02-06 02:33:42 +0000
@@ -14,11 +14,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Remove the last revision from the history of the current branch.
-"""
+"""Remove the last revision from the history of the current branch."""
+
+# TODO: make the guts of this methods on tree, branch.
 
 import os
 
+from bzrlib.branch import Branch
 from bzrlib.errors import BoundBranchOutOfDate
 
 
@@ -51,10 +53,14 @@
             raise BoundBranchOutOfDate(branch, master)
         if revno is None:
             revno = len(rh)
+        old_revno, old_tip = branch.last_revision_info()
+        new_revno = revno -1
 
         files_to_remove = []
         for r in range(revno-1, len(rh)):
             rev_id = rh.pop()
+            # NB: performance would be better using the revision graph rather
+            # than the whole revision.
             rev = branch.repository.get_revision(rev_id)
             # When we finish popping off the pending merges, we want
             # them to stay in the order that they used to be.
@@ -70,10 +76,19 @@
             if master is not None:
                 master.set_revision_history(rh)
             branch.set_revision_history(rh)
+            new_tip = branch.last_revision()
+            if master is None:
+                hook_local = None
+                hook_master = branch
+            else:
+                hook_local = branch
+                hook_master = master
+            for hook in Branch.hooks['post_uncommit']:
+                hook(hook_local, hook_master, old_revno, old_tip, new_revno,
+                    new_tip)
             if tree is not None:
-                branch_tip = branch.last_revision()
-                if branch_tip is not None:
-                    parents = [branch.last_revision()]
+                if new_tip is not None:
+                    parents = [new_tip]
                 else:
                     parents = []
                 parents.extend(reversed(pending_merges))

